Package: digest
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Antoine", "Lucas", role="ctb"),
             person("Jarek", "Tuszynski", role="ctb"),
             person("Henrik", "Bengtsson", role="ctb", comment = c(ORCID = "0000-0002-7579-5165")),
             person("Simon", "Urbanek", role="ctb", comment = c(ORCID = "0000-0003-2297-1732")),
             person("Mario", "Frasca", role="ctb"),
             person("Bryan", "Lewis", role="ctb"),
             person("Murray", "Stokely", role="ctb"),
             person("Hannes", "Muehleisen", role="ctb"),
             person("Duncan", "Murdoch", role="ctb"),
             person("Jim", "Hester", role="ctb"),
             person("Wush", "Wu", role="ctb", comment = c(ORCID = "0000-0001-5180-0567")),
             person("Qiang", "Kou", role="ctb", comment = c(ORCID = "0000-0001-6786-5453")),
             person("Thierry", "Onkelinx", role="ctb", comment = c(ORCID = "0000-0001-8804-4216")),
             person("Michel", "Lang", role="ctb", comment = c(ORCID = "0000-0001-9754-0393")),
             person("Viliam", "Simko", role="ctb"),
             person("Kurt", "Hornik", role="ctb", comment = c(ORCID = "0000-0003-4198-9911")),
             person("Radford", "Neal", role="ctb", comment = c(ORCID = "0000-0002-2473-3407")),
             person("Kendon", "Bell", role="ctb", comment = c(ORCID = "0000-0002-9093-8312")),
             person("Matthew", "de Queljoe", role="ctb"),
             person("Dmitry", "Selivanov", role="ctb"),
             person("Ion", "Suruceanu", role="ctb"),
             person("Bill", "Denney", role="ctb"),
             person("Dirk", "Schumacher", role="ctb"),
             person("András", "Svraka", role="ctb"),
             person("Sergey", "Fedorov", role="ctb"),
             person("Will", "Landau", role="ctb", comment = c(ORCID = "0000-0003-1878-3253")),
             person("Floris", "Vanderhaeghe", role="ctb", comment = c(ORCID = "0000-0002-6378-6229")),
             person("Kevin", "Tappe", role="ctb"),
             person("Harris", "McGehee", role="ctb"),
             person("Tim", "Mastny", role="ctb"),
             person("Aaron", "Peikert", role="ctb", comment = c(ORCID = "0000-0001-7813-818X")),
             person("Mark", "van der Loo", role="ctb", comment = c(ORCID = "0000-0002-9807-4686")),
             person("Chris", "Muir", role="ctb", comment = c(ORCID = "0000-0003-2555-3878")),
             person("Moritz", "Beller", role="ctb", comment = c(ORCID = "0000-0003-4852-0526")),
             person("Sebastian", "Campbell", role="ctb"),
             person("Winston", "Chang", role="ctb", comment = c(ORCID = "0000-0002-1576-2126")),
             person("Dean", "Attali", role="ctb", comment = c(ORCID = "0000-0002-5645-3493")),
             person("Michael", "Chirico", role="ctb", comment = c(ORCID = "0000-0003-0787-087X")),
             person("Kevin", "Ushey", role="ctb"))
Version: 0.6.37
Date: 2024-08-19
Title: Create Compact Hash Digests of R Objects
Description: Implementation of a function 'digest()' for the creation of hash
 digests of arbitrary R objects (using the 'md5', 'sha-1', 'sha-256', 'crc32',
 'xxhash', 'murmurhash', 'spookyhash', 'blake3', 'crc32c', 'xxh3_64', and 'xxh3_128'
 algorithms) permitting easy comparison of R language objects, as well as functions
 such as'hmac()' to create hash-based message authentication code. Please note that
 this package is not meant to be deployed for cryptographic purposes for which more
 comprehensive (and widely tested) libraries such as 'OpenSSL' should be used.
URL: https://github.com/eddelbuettel/digest,
        https://dirk.eddelbuettel.com/code/digest.html
BugReports: https://github.com/eddelbuettel/digest/issues
Depends: R (>= 3.3.0)
Imports: utils
License: GPL (>= 2)
Suggests: tinytest, simplermarkdown
VignetteBuilder: simplermarkdown
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-08-19 12:16:05 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (<https://orcid.org/0000-0001-6419-907X>),
  Antoine Lucas [ctb],
  Jarek Tuszynski [ctb],
  Henrik Bengtsson [ctb] (<https://orcid.org/0000-0002-7579-5165>),
  Simon Urbanek [ctb] (<https://orcid.org/0000-0003-2297-1732>),
  Mario Frasca [ctb],
  Bryan Lewis [ctb],
  Murray Stokely [ctb],
  Hannes Muehleisen [ctb],
  Duncan Murdoch [ctb],
  Jim Hester [ctb],
  Wush Wu [ctb] (<https://orcid.org/0000-0001-5180-0567>),
  Qiang Kou [ctb] (<https://orcid.org/0000-0001-6786-5453>),
  Thierry Onkelinx [ctb] (<https://orcid.org/0000-0001-8804-4216>),
  Michel Lang [ctb] (<https://orcid.org/0000-0001-9754-0393>),
  Viliam Simko [ctb],
  Kurt Hornik [ctb] (<https://orcid.org/0000-0003-4198-9911>),
  Radford Neal [ctb] (<https://orcid.org/0000-0002-2473-3407>),
  Kendon Bell [ctb] (<https://orcid.org/0000-0002-9093-8312>),
  Matthew de Queljoe [ctb],
  Dmitry Selivanov [ctb],
  Ion Suruceanu [ctb],
  Bill Denney [ctb],
  Dirk Schumacher [ctb],
  András Svraka [ctb],
  Sergey Fedorov [ctb],
  Will Landau [ctb] (<https://orcid.org/0000-0003-1878-3253>),
  Floris Vanderhaeghe [ctb] (<https://orcid.org/0000-0002-6378-6229>),
  Kevin Tappe [ctb],
  Harris McGehee [ctb],
  Tim Mastny [ctb],
  Aaron Peikert [ctb] (<https://orcid.org/0000-0001-7813-818X>),
  Mark van der Loo [ctb] (<https://orcid.org/0000-0002-9807-4686>),
  Chris Muir [ctb] (<https://orcid.org/0000-0003-2555-3878>),
  Moritz Beller [ctb] (<https://orcid.org/0000-0003-4852-0526>),
  Sebastian Campbell [ctb],
  Winston Chang [ctb] (<https://orcid.org/0000-0002-1576-2126>),
  Dean Attali [ctb] (<https://orcid.org/0000-0002-5645-3493>),
  Michael Chirico [ctb] (<https://orcid.org/0000-0003-0787-087X>),
  Kevin Ushey [ctb]
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2024-08-19 14:10:07 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-25 21:35:12 UTC; unix
Archs: digest.so.dSYM

## package has a dynamic library
useDynLib(digest, digest_impl=digest, vdigest_impl=vdigest, digest2int_impl=digest2int, AESinit, AESencryptECB, AESdecryptECB, spookydigest_impl, is_little_endian, is_big_endian, .registration=TRUE)

importFrom(utils, packageVersion)

## and exported functions
export(AES,
       digest,
       digest2int,
       getVDigest,
       sha1,
       sha1_attr_digest,
       sha1_digest,
       hmac,
       makeRaw)

S3method(print, AES)

S3method(sha1, anova)
S3method(sha1, array)
S3method(sha1, call)
S3method(sha1, character)
S3method(sha1, complex)
S3method(sha1, data.frame)
S3method(sha1, Date)
S3method(sha1, default)
S3method(sha1, integer)
S3method(sha1, factor)
S3method(sha1, formula)
S3method(sha1, "function")
S3method(sha1, logical)
S3method(sha1, list)
S3method(sha1, matrix)
S3method(sha1, name)
S3method(sha1, NULL)
S3method(sha1, numeric)
S3method(sha1, pairlist)
S3method(sha1, POSIXct)
S3method(sha1, POSIXlt)
S3method(sha1, raw)
S3method(sha1, "(")

S3method(makeRaw, default)
S3method(makeRaw, digest)
S3method(makeRaw, character)
S3method(makeRaw, raw)

NAME
    Digest - Modules that calculate message digests

SYNOPSIS
      $md5  = Digest->new("MD5");
      $sha1 = Digest->new("SHA-1");
      $sha256 = Digest->new("SHA-256");
      $sha384 = Digest->new("SHA-384");
      $sha512 = Digest->new("SHA-512");

      $hmac = Digest->HMAC_MD5($key);

DESCRIPTION
    The "Digest::" modules calculate digests, also called "fingerprints" or
    "hashes", of some data, called a message. The digest is (usually) some
    small/fixed size string. The actual size of the digest depend of the
    algorithm used. The message is simply a sequence of arbitrary bytes or
    bits.

    An important property of the digest algorithms is that the digest is
    *likely* to change if the message change in some way. Another property
    is that digest functions are one-way functions, that is it should be
    *hard* to find a message that correspond to some given digest.
    Algorithms differ in how "likely" and how "hard", as well as how
    efficient they are to compute.

    Note that the properties of the algorithms change over time, as the
    algorithms are analyzed and machines grow faster. If your application
    for instance depends on it being "impossible" to generate the same
    digest for a different message it is wise to make it easy to plug in
    stronger algorithms as the one used grow weaker. Using the interface
    documented here should make it easy to change algorithms later.

    All "Digest::" modules provide the same programming interface. A
    functional interface for simple use, as well as an object oriented
    interface that can handle messages of arbitrary length and which can
    read files directly.

    The digest can be delivered in three formats:

    *binary*
            This is the most compact form, but it is not well suited for
            printing or embedding in places that can't handle arbitrary
            data.

    *hex*   A twice as long string of lowercase hexadecimal digits.

    *base64*
            A string of portable printable characters. This is the base64
            encoded representation of the digest with any trailing padding
            removed. The string will be about 30% longer than the binary
            version. MIME::Base64 tells you more about this encoding.

    The functional interface is simply importable functions with the same
    name as the algorithm. The functions take the message as argument and
    return the digest. Example:

      use Digest::MD5 qw(md5);
      $digest = md5($message);

    There are also versions of the functions with "_hex" or "_base64"
    appended to the name, which returns the digest in the indicated form.

OO INTERFACE
    The following methods are available for all "Digest::" modules:

    $ctx = Digest->XXX($arg,...)
    $ctx = Digest->new(XXX => $arg,...)
    $ctx = Digest::XXX->new($arg,...)
        The constructor returns some object that encapsulate the state of
        the message-digest algorithm. You can add data to the object and
        finally ask for the digest. The "XXX" should of course be replaced
        by the proper name of the digest algorithm you want to use.

        The two first forms are simply syntactic sugar which automatically
        load the right module on first use. The second form allow you to use
        algorithm names which contains letters which are not legal perl
        identifiers, e.g. "SHA-1". If no implementation for the given
        algorithm can be found, then an exception is raised.

        To know what arguments (if any) the constructor takes (the
        "$args,..." above) consult the docs for the specific digest
        implementation.

        If new() is called as an instance method (i.e. $ctx->new) it will
        just reset the state the object to the state of a newly created
        object. No new object is created in this case, and the return value
        is the reference to the object (i.e. $ctx).

    $other_ctx = $ctx->clone
        The clone method creates a copy of the digest state object and
        returns a reference to the copy.

    $ctx->reset
        This is just an alias for $ctx->new.

    $ctx->add( $data )
    $ctx->add( $chunk1, $chunk2, ... )
        The string value of the $data provided as argument is appended to
        the message we calculate the digest for. The return value is the
        $ctx object itself.

        If more arguments are provided then they are all appended to the
        message, thus all these lines will have the same effect on the state
        of the $ctx object:

          $ctx->add("a"); $ctx->add("b"); $ctx->add("c");
          $ctx->add("a")->add("b")->add("c");
          $ctx->add("a", "b", "c");
          $ctx->add("abc");

        Most algorithms are only defined for strings of bytes and this
        method might therefore croak if the provided arguments contain chars
        with ordinal number above 255.

    $ctx->addfile( $io_handle )
        The $io_handle is read until EOF and the content is appended to the
        message we calculate the digest for. The return value is the $ctx
        object itself.

        The addfile() method will croak() if it fails reading data for some
        reason. If it croaks it is unpredictable what the state of the $ctx
        object will be in. The addfile() method might have been able to read
        the file partially before it failed. It is probably wise to discard
        or reset the $ctx object if this occurs.

        In most cases you want to make sure that the $io_handle is in
        "binmode" before you pass it as argument to the addfile() method.

    $ctx->add_bits( $data, $nbits )
    $ctx->add_bits( $bitstring )
        The add_bits() method is an alternative to add() that allow partial
        bytes to be appended to the message. Most users can just ignore this
        method since typical applications involve only whole-byte data.

        The two argument form of add_bits() will add the first $nbits bits
        from $data. For the last potentially partial byte only the high
        order "$nbits % 8" bits are used. If $nbits is greater than
        "length($data) * 8", then this method would do the same as
        "$ctx->add($data)".

        The one argument form of add_bits() takes a $bitstring of "1" and
        "0" chars as argument. It's a shorthand for
        "$ctx->add_bits(pack("B*", $bitstring), length($bitstring))".

        The return value is the $ctx object itself.

        This example shows two calls that should have the same effect:

           $ctx->add_bits("111100001010");
           $ctx->add_bits("\xF0\xA0", 12);

        Most digest algorithms are byte based and for these it is not
        possible to add bits that are not a multiple of 8, and the
        add_bits() method will croak if you try.

    $ctx->digest
        Return the binary digest for the message.

        Note that the "digest" operation is effectively a destructive,
        read-once operation. Once it has been performed, the $ctx object is
        automatically "reset" and can be used to calculate another digest
        value. Call $ctx->clone->digest if you want to calculate the digest
        without resetting the digest state.

    $ctx->hexdigest
        Same as $ctx->digest, but will return the digest in hexadecimal
        form.

    $ctx->b64digest
        Same as $ctx->digest, but will return the digest as a base64 encoded
        string without padding.

    $ctx->base64_padded_digest
        Same as $ctx->digest, but will return the digest as a base64 encoded
        string.

Digest speed
    This table should give some indication on the relative speed of
    different algorithms. It is sorted by throughput based on a benchmark
    done with of some implementations of this API:

     Algorithm      Size    Implementation                  MB/s

     MD4            128     Digest::MD4 v1.3               165.0
     MD5            128     Digest::MD5 v2.33               98.8
     SHA-256        256     Digest::SHA2 v1.1.0             66.7
     SHA-1          160     Digest::SHA v4.3.1              58.9
     SHA-1          160     Digest::SHA1 v2.10              48.8
     SHA-256        256     Digest::SHA v4.3.1              41.3
     Haval-256      256     Digest::Haval256 v1.0.4         39.8
     SHA-384        384     Digest::SHA2 v1.1.0             19.6
     SHA-512        512     Digest::SHA2 v1.1.0             19.3
     SHA-384        384     Digest::SHA v4.3.1              19.2
     SHA-512        512     Digest::SHA v4.3.1              19.2
     Whirlpool      512     Digest::Whirlpool v1.0.2        13.0
     MD2            128     Digest::MD2 v2.03                9.5

     Adler-32        32     Digest::Adler32 v0.03            1.3
     CRC-16          16     Digest::CRC v0.05                1.1
     CRC-32          32     Digest::CRC v0.05                1.1
     MD5            128     Digest::Perl::MD5 v1.5           1.0
     CRC-CCITT       16     Digest::CRC v0.05                0.8

    These numbers was achieved Apr 2004 with ActivePerl-5.8.3 running under
    Linux on a P4 2.8 GHz CPU. The last 5 entries differ by being pure perl
    implementations of the algorithms, which explains why they are so slow.

SEE ALSO
    Digest::Adler32, Digest::CRC, Digest::Haval256, Digest::HMAC,
    Digest::MD2, Digest::MD4, Digest::MD5, Digest::SHA, Digest::SHA1,
    Digest::SHA2, Digest::Whirlpool

    New digest implementations should consider subclassing from
    Digest::base.

    MIME::Base64

    http://en.wikipedia.org/wiki/Cryptographic_hash_function

AUTHOR
    Gisle Aas <gisle@aas.no>

    The "Digest::" interface is based on the interface originally developed
    by Neil Winton for his "MD5" module.

    This library is free software; you can redistribute it and/or modify it
    under the same terms as Perl itself.

        Copyright 1998-2006 Gisle Aas.
        Copyright 1995,1996 Neil Winton.

# digest: Hash representations of arbitrary R objects

[![ci](https://github.com/eddelbuettel/digest/workflows/ci/badge.svg)](https://github.com/eddelbuettel/digest/actions?query=workflow%3Aci)
[![License](https://img.shields.io/badge/license-GPL%20%28%3E=%202%29-brightgreen.svg?style=flat)](https://www.r-project.org/Licenses/GPL-2)
[![CRAN](https://www.r-pkg.org/badges/version/digest)](https://cran.r-project.org/package=digest)
[![Dependencies](https://tinyverse.netlify.app/badge/digest)](https://cran.r-project.org/package=digest)
[![Debian package](https://img.shields.io/debian/v/r-cran-digest/sid?color=green)](https://packages.debian.org/sid/r-cran-digest)
[![r-universe](https://eddelbuettel.r-universe.dev/badges/digest)](https://eddelbuettel.r-universe.dev/digest)
[![Downloads (monthly)](https://cranlogs.r-pkg.org/badges/digest?color=brightgreen)](https://www.r-pkg.org:443/pkg/digest)
[![Downloads (total)](https://cranlogs.r-pkg.org/badges/grand-total/digest?color=brightgreen)](https://www.r-pkg.org:443/pkg/digest)
[![CRAN use](https://jangorecki.gitlab.io/rdeps/digest/CRAN_usage.svg?sanitize=true)](https://cran.r-project.org/package=digest)
[![CRAN indirect](https://jangorecki.gitlab.io/rdeps/digest/indirect_usage.svg?sanitize=true)](https://cran.r-project.org/package=digest)
[![BioConductor use](https://jangorecki.gitlab.io/rdeps/digest/BioC_usage.svg?sanitize=true)](https://cran.r-project.org/package=digest)
[![Code Coverage](https://img.shields.io/codecov/c/github/eddelbuettel/digest/master.svg)](https://app.codecov.io/gh/eddelbuettel/digest)
[![Last Commit](https://img.shields.io/github/last-commit/eddelbuettel/digest)](https://github.com/eddelbuettel/digest)
[![Documentation](https://img.shields.io/badge/documentation-is_here-blue)](https://eddelbuettel.github.io/digest/)

### Overview

The digest package provides a principal function `digest()` for the creation
of hash digests of arbitrary R objects (using the md5, sha-1, sha-256, crc32,
xxhash, murmurhash, spookyhash, blake3, crc32c, xxh3\_64, and xxh3\_128
algorithms) permitting easy comparison of R language objects.

Extensive documentation is available at the [package documentation site](https://eddelbuettel.github.io/digest/).

#### Examples

As R can serialize any object, we can run `digest()` on any object:

```r
R> library(digest)
R> digest(trees)
[1] "12412cbfa6629c5c80029209b2717f08"
R> digest(lm(log(Height) ~ log(Girth), data=trees))
[1] "e25b62de327d079b3ccb98f3e96987b1"
R> digest(summary(lm(log(Height) ~ log(Girth), data=trees)))
[1] "86c8c979ee41a09006949e2ad95feb41"
R>
```

By using the hash sum, which is very likely to be unique, to identify an
underlying object or calculation, one can easily implement caching strategies.
This is a common use of the digest package.

#### Other Functions

A small number of additional functions is available:

- `sha1()` for numerically stable hashsums,
- `hmac()` for hashed message authentication codes based on a key,
- `AES()` for Advanced Encryption Standard block ciphers,
- `getVDigest()` as a function generator for vectorised versions.

### Note

Please note that this package is not meant to be deployed for
cryptographic purposes. More comprehensive and widely tested
libraries such as OpenSSL should be used instead.

### Installation

The package is on [CRAN](https://cran.r-project.org) and can be installed
via a standard

```r
install.packages("digest")
```

### Continued Testing

As we rely on the [tinytest](https://cran.r-project.org/package=tinytest) package, the
already-installed package can also be verified via

```r
tinytest::test_package("digest")
```

at any later point.

### Author

Dirk Eddelbuettel, with contributions by Antoine Lucas, Jarek Tuszynski,
Henrik Bengtsson, Simon Urbanek, Mario Frasca, Bryan Lewis, Murray Stokely,
Hannes Muehleisen, Duncan Murdoch, Jim Hester, Wush Wu, Qiang Kou, Thierry
Onkelinx, Michel Lang, Viliam Simko, Kurt Hornik, Radford Neal, Kendon Bell,
Matthew de Queljoe, Ion Suruceanu, Bill Denney, Dirk Schumacher, Winston
Chang, Dean Attali, and Michael Chirico.

### License

GPL (>= 2)

The *digest* utility calculates message digests of files or, if no file is
specified, standard input. The set of supported digests depends on the
current Python interpreter and the version of OpenSSL present on the
system. However, at a minimum, *digest* supports the following algorithms:

* md5: The MD5 algorithm
* sha1: The SHA1 algorithm
* sha224: The SHA224 algorithm
* sha256: The SHA256 algorithm
* sha384: The SHA384 algorithm
* sha512: The SHA512 algorithm

This program is modeled on the FreeBSD [*digest*][] port (written by
Alistair G. Crooks). This Python version is an independently implemented
program based on the manual page and on output from Crooks' program.

For more information, please see the [home page][].

[*digest*]: http://www.freebsd.org/cgi/url.cgi?ports/security/digest/pkg-descr
[home page]: http://software.clapper.org/digest/

# faq7_31
# FAQ 7.31
a0 <- 2
b <- sqrt(a0)
a1 <- b ^ 2
identical(a0, a1)
a0 - a1
a <- c(a0, a1)
# hexadecimal representation
sprintf("%a", a)

# faq7_31digest
library(digest)
# different hashes with digest
sapply(a, digest, algo = "sha1")
# same hash with sha1 with default digits (14)
sapply(a, sha1)
# larger digits can lead to different hashes
sapply(a, sha1, digits = 15)
# decreasing the number of digits gives a stronger truncation
# the hash will change when then truncation gives a different result
# case where truncating gives same hexadecimal value
sapply(a, sha1, digits = 13)
sapply(a, sha1, digits = 10)
# case where truncating gives different hexadecimal value
c(sha1(pi), sha1(pi, digits = 13), sha1(pi, digits = 10))

# sha1_lm_sum
# taken from the help file of lm.influence
lm_SR <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LifeCycleSavings)
lm_sum <- summary(lm_SR)
class(lm_sum)
# str() gives the structure of the lm object
str(lm_sum)
# extract the coefficients and their standard error
coef_sum <- coef(lm_sum)[, c("Estimate", "Std. Error")]
# extract sigma
sigma <- lm_sum$sigma
# check the class of each component
class(coef_sum)
class(sigma)
# sha1() has methods for both matrix and numeric
# because the values originate from floating point arithmetic it is better to use a low number of digits
sha1(coef_sum, digits = 4)
sha1(sigma, digits = 4)
# we want a single hash
# combining the components in a list is a solution that works
sha1(list(coef_sum, sigma), digits = 4)
# now turn everything into an S3 method
#   - a function with name "sha1.classname"
#   - must have the same arguments as sha1()
sha1.summary.lm <- function(x, digits = 4, zapsmall = 7){
    coef_sum <- coef(x)[, c("Estimate", "Std. Error")]
    sigma <- x$sigma
    combined <- list(coef_sum, sigma)
    sha1(combined, digits = digits, zapsmall = zapsmall)
}
sha1(lm_sum)

# try an altered dataset
LCS2 <- LifeCycleSavings[rownames(LifeCycleSavings) != "Zambia", ]
lm_SR2 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LCS2)
sha1(summary(lm_SR2))

# sha1_lm
class(lm_SR)
# str() gives the structure of the lm object
str(lm_SR)
# extract the model and the terms
lm_model <- lm_SR$model
lm_terms <- lm_SR$terms
# check their class
class(lm_model) # handled by sha1()
class(lm_terms) # not handled by sha1()
# define a method for formula
sha1.formula <- function(x, digits = 14, zapsmall = 7, ..., algo = "sha1"){
    sha1(as.character(x), digits = digits, zapsmall = zapsmall, algo = algo)
}
sha1(lm_terms)
sha1(lm_model)
# define a method for lm
sha1.lm <- function(x, digits = 14, zapsmall = 7, ..., algo = "sha1"){
    lm_model <- x$model
    lm_terms <- x$terms
    combined <- list(lm_model, lm_terms)
    sha1(combined, digits = digits, zapsmall = zapsmall, ..., algo = algo)
}
sha1(lm_SR)
sha1(lm_SR2)

library(digest)

#vectorisation
md5 <- getVDigest()
digest2 <- base::Vectorize(digest)
x <- rep(letters, 1e3)
rbenchmark::benchmark(
    vdigest = md5(x, serialize = FALSE),
    Vectorize = digest2(x, serialize = FALSE),
    vapply = vapply(x, digest, character(1), serialize = FALSE),
    replications = 5
)[,1:4]
all(md5(x, serialize=FALSE) == digest2(x, serialize=FALSE))
all(md5(x, serialize=FALSE) == vapply(x, digest, character(1), serialize = FALSE))

#repeated calls
stretch_key <- function(d, n) {
    md5 <- getVDigest()
    for (i in seq_len(n))
        d <- md5(d, serialize = FALSE)
    d
}

stretch_key2 <- function(d, n) {
    for (i in seq_len(n))
        d <- digest(d, serialize = FALSE)
    d
}
rbenchmark::benchmark(
    vdigest = stretch_key('abc123', 65e3),
    plaindigest = stretch_key2('abc123', 65e3),
    replications = 10
)[,1:4]
stretch_key('abc123', 65e3) == stretch_key2('abc123', 65e3)

##  digest -- hash digest functions for R
##
##  Copyright (C) 2003 - 2024  Dirk Eddelbuettel <edd@debian.org>
##  Copyright (C) 2009 - 2019  Henrik Bengtsson
##  Copyright (C) 2012 - 2019  Hannes Muehleisen
##  Copyright (C) 2014 - 2019  Jim Hester
##  Copyright (C) 2019         Kendon Bell
##  Copyright (C) 2019         Matthew de Queljoe
##
##  This file is part of digest.
##
##  digest is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  digest is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with digest.  If not, see <http://www.gnu.org/licenses/>.

digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
                                  "xxhash32", "xxhash64", "murmur32",
                                  "spookyhash", "blake3", "crc32c",
                                  "xxh3_64", "xxh3_128"),
                   serialize=TRUE,
                   file=FALSE,
                   length=Inf,
                   skip="auto",
                   ascii=FALSE,
                   raw=FALSE,
                   seed=0,
                   errormode=c("stop","warn","silent"),
                   serializeVersion=.getSerializeVersion()) {

    # Explicitly specify choices; this is much faster than having match.arg()
    # infer them from the function's formals.
    algo <- match.arg(algo, c("md5", "sha1", "crc32", "sha256", "sha512",
                              "xxhash32", "xxhash64", "murmur32",
                              "spookyhash", "blake3", "crc32c",
                              "xxh3_64", "xxh3_128"))
    errormode <- match.arg(errormode, c("stop", "warn", "silent"))

    if (is.infinite(length)) {
        length <- -1               # internally we use -1 for infinite len
    }

    if (is.character(file) && missing(object)) {
        object <- file                  # nocov
        file <- TRUE                  	# nocov
    }

    is_streaming_algo <- algo == "spookyhash"

    if (is_streaming_algo && !serialize) {
        .errorhandler(paste0(algo, " algorithm is not available without serialization."),  # #nocov
                      mode=errormode)                                                      # #nocov
    }

    if (serialize && !file) {
        if (!is_streaming_algo) {
            ## support the 'nosharing' option in pqR's serialize()
            object <- if (.hasNoSharing())
                serialize (object, connection=NULL, ascii=ascii,
                           nosharing=TRUE, version=serializeVersion)
            else
                serialize (object, connection=NULL, ascii=ascii,
                           version=serializeVersion)
        }
        ## we support raw vectors, so no mangling of 'object' is necessary
        ## regardless of R version
        ## skip="auto" - skips the serialization header [SU]
        if (is.character(skip) && skip == "auto")
            skip <- set_skip(object, ascii)

    } else if (!is.character(object) && !inherits(object,"raw") &&
               !is_streaming_algo) {
        return(.errorhandler(paste("Argument object must be of type character",		    # #nocov
                                   "or raw vector if serialize is FALSE"), mode=errormode)) # #nocov
    }
    if (file && !is.character(object))
        return(.errorhandler("file=TRUE can only be used with a character object",          # #nocov
                             mode=errormode))                                               # #nocov

    if (file && is_streaming_algo)
        return(.errorhandler(paste0(algo, " algorithm can not be used with files."),        # #nocov
                             mode=errormode))                                               # #nocov

    ## HB 14 Mar 2007:  null op, only turned to char if alreadt char
    ##if (!inherits(object,"raw"))
    ##  object <- as.character(object)
    algoint <- algo_int(algo)
    if (file) {
        algoint <- algoint+100
        object <- path.expand(object)
        if (.isWindows()) object <- enc2utf8(object)
        check_file(object, errormode)
    }
    ## if skip is auto (or any other text for that matter), we just turn it
    ## into 0 because auto should have been converted into a number earlier
    ## if it was valid [SU]
    if (is.character(skip)) skip <- 0
    if (!is_streaming_algo) {
        val <- .Call(digest_impl,
                     object,
                     as.integer(algoint),
                     as.integer(length),
                     as.integer(skip),
                     as.integer(raw),
                     as.integer(seed))
    } else if (algo == "spookyhash"){
        # 0s are the seeds. They are included to enable testing against fastdigest.
        val <- paste(.Call(spookydigest_impl, object, skip, 0, 0, serializeVersion, NULL), collapse="")
    }

    ## crc32 output was not guaranteed to be eight chars long, which we corrected
    ## this allows to get the old behaviour back for compatibility
    if ((algoint == 3 || algoint == 103) && .getCRC32PreferOldOutput()) {
        val <- sub("^0+", "", val)
    }

    return(val)
}

## utility functions used by digest() and  getVDigest() below

.errorhandler <- function(txt, obj="", mode="stop") {
    if (mode == "stop") {                                                                # nocov start
        stop(txt, obj, call.=FALSE)
    } else if (mode == "warn") {
        warning(txt, obj, call.=FALSE)
        return(invisible(NA))
    } else {
        return(invisible(NULL))                                                          # nocov end
    }
}

algo_int <- function(algo)
    switch(
        algo,
        md5 = 1,
        sha1 = 2,
        crc32 = 3,
        sha256 = 4,
        sha512 = 5,
        xxhash32 = 6,
        xxhash64 = 7,
        murmur32 = 8,
        spookyhash = 9,
        blake3 = 10,
        crc32c = 11,
        xxh3_64 = 12,
        xxh3_128 = 13
    )

## HB 14 Mar 2007:
## Exclude serialization header (non-data dependent bytes but R
## version specific).  In ASCII, the header consists of for rows
## ending with a newline ('\n').  We need to skip these.
## The end of 4th row is *typically* within the first 18 bytes
set_skip <- function(object, ascii){
    if (!ascii)
        return(14)
    ## Was: skip <- if (ascii) 18 else 14
    which(object[1:30] == as.raw(10))[4]         					# nocov
}

check_file <- function(object, errormode){
    if (!file.exists(object)) {
        return(.errorhandler("The file does not exist: ", object, mode=errormode)) 	# nocov start
    }
    if (!isTRUE(!file.info(object)$isdir)) {
        return(.errorhandler("The specified pathname is not a file: ",
                             object, mode=errormode))
    }
    if (file.access(object, 4)) {
        return(.errorhandler("The specified file is not readable: ",
                             object, mode=errormode))                  			# #nocov end
    }
}

isWindows <- Sys.info()[["sysname"]] == "Windows"
if (!isWindows) exit_file("Skipping test on non-Windows platform")

d <- tempfile()
dir.create(d)

# A file path not representable in latin-1
f <- file.path(d, "tricky-ő")

# We need to use a binary mode connection so the newlines
# are consistent across platforms
con <- file(f, open = "wb")
writeLines("foobar", con = con)
close(con)

expect_identical(digest::digest(file = f), "14758f1afd44c09b7992073ccf00b43d")

expect_identical(digest::digest(f, file = TRUE), "14758f1afd44c09b7992073ccf00b43d")

vd <- getVDigest()

expect_identical(vd(file = f), "14758f1afd44c09b7992073ccf00b43d")

expect_identical(vd(f, file = TRUE), "14758f1afd44c09b7992073ccf00b43d")

unlink(d, recursive = TRUE)

## tests for digest, taken from the examples in the manual page

suppressMessages(library(digest))

## Standard RFC 1321 test vectors
md5Input <-
    c("",
      "a",
      "abc",
      "message digest",
      "abcdefghijklmnopqrstuvwxyz",
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
      paste("12345678901234567890123456789012345678901234567890123456789012",
            "345678901234567890", sep=""))
md5Output <-
    c("d41d8cd98f00b204e9800998ecf8427e",
      "0cc175b9c0f1b6a831c399e269772661",
      "900150983cd24fb0d6963f7d28e17f72",
      "f96b697d7cb7938d525a2f31aaf161d0",
      "c3fcd3d76192e4007dfb496cca67e13b",
      "d174ab98d277d9f5a5611c2c9f419d9f",
      "57edf4a22be3c955ac49da2e2107b67a")

for (i in seq(along.with=md5Input)) {
    md5 <- digest(md5Input[i], serialize=FALSE)
    expect_true(identical(md5, md5Output[i]))
    #cat(md5, "\n")
}

md5 <- getVDigest()
expect_identical(md5(md5Input, serialize = FALSE), md5Output)

expect_identical(digest(NULL),
                 md5(NULL))
expect_identical(digest(character(0)),
                 md5(character(0)))
expect_identical(digest(list("abc")),
                 md5(list(list("abc"))))
expect_identical(digest(list(NULL)),
                 md5(list(list(NULL))))
expect_identical(digest(character(0), serialize = FALSE),
                 md5(character(0), serialize = FALSE))


## md5 raw output test
for (i in seq(along.with=md5Input)) {
    md5 <- digest(md5Input[i], serialize=FALSE, raw=TRUE)
    md5 <- gsub(" ","",capture.output(cat(md5)))
    expect_true(identical(md5, md5Output[i]))
    #cat(md5, "\n")
}

sha1Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      NULL)
sha1Output <-
    c("a9993e364706816aba3e25717850c26c9cd0d89d",
      "84983e441c3bd26ebaae4aa1f95129e5e54670f1",
      "34aa973cd4c4daa4f61eeb2bdbad27316534016f")

for (i in seq(along.with=sha1Input)) {
    sha1 <- digest(sha1Input[i], algo="sha1", serialize=FALSE)
    expect_true(identical(sha1, sha1Output[i]))
    #cat(sha1, "\n")
}

sha1 <- getVDigest(algo = 'sha1')
expect_identical(sha1(sha1Input, serialize = FALSE), sha1Output[1:2])

## sha1 raw output test
for (i in seq(along.with=sha1Input)) {
    sha1 <- digest(sha1Input[i], algo="sha1", serialize=FALSE, raw=TRUE)
    #print(sha1)
    sha1 <- gsub(" ","",capture.output(cat(sha1)))
    #print(sha1)
    #print(sha1Output[i])
    expect_true(identical(sha1, sha1Output[i]))
    #cat(sha1, "\n")
}

## sha512 test
sha512Input <-c(
    "",
    "The quick brown fox jumps over the lazy dog."
    )
sha512Output <- c(
    "cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e",
    "91ea1245f20d46ae9a037a989f54f1f790f0a47607eeb8a14d12890cea77a1bbc6c7ed9cf205e67b7f2b8fd4c7dfd3a7a8617e45f3c463d481c7e586c39ac1ed")

for (i in seq(along.with=sha512Input)) {
    sha512 <- digest(sha512Input[i], algo="sha512", serialize=FALSE)
    expect_true(identical(sha512, sha512Output[i]))
    #cat(sha512, "\n")
}

sha512 <- getVDigest(algo = 'sha512')
expect_identical(sha512(sha512Input, serialize = FALSE), sha512Output[1:2])

## sha512 raw output test
for (i in seq(along.with=sha512Input)) {
    sha512 <- digest(sha512Input[i], algo="sha512", serialize=FALSE, raw=TRUE)
    #print(sha512)

    sha512 <- gsub(" ","",capture.output(cat(sha512)))
    #print(sha512)
    #print(sha512Output[i])
    expect_true(identical(sha512, sha512Output[i]))
    #cat(sha512, "\n")
}

crc32Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      NULL)
crc32Output <-
    c("352441c2",
      "171a3f5f",
      "2ef80172")

for (i in seq(along.with=crc32Input)) {
    crc32 <- digest(crc32Input[i], algo="crc32", serialize=FALSE)
    expect_true(identical(crc32, crc32Output[i]))
    #cat(crc32, "\n")
}

crc32 <- getVDigest(algo = 'crc32')
expect_identical(crc32(crc32Input, serialize = FALSE), crc32Output[1:2])


## one of the FIPS-
sha1 <- digest("abc", algo="sha1", serialize=FALSE)
expect_true(identical(sha1, "a9993e364706816aba3e25717850c26c9cd0d89d"))

## This one seems to give slightly different output depending on the R version used
##
##                                      # example of a digest of a standard R list structure
## cat(digest(list(LETTERS, data.frame(a=letters[1:5],
##                                     b=matrix(1:10,
##                                     ncol=2)))), "\n")

## these outputs were calculated using xxh32sum
## [ Correction:  These reproduce via the Python xxhash package and its hexdigest() output
##   but not the xxh64sum command-line tool as the original comment here implies. ]
xxhash32Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
xxhash32Output <-
    c("32d153ff",
      "89ea60c3",
      "02cc5d05")

for (i in seq(along.with=xxhash32Input)) {
    xxhash32 <- digest(xxhash32Input[i], algo="xxhash32", serialize=FALSE)
    #cat(xxhash32, "\n")
    expect_true(identical(xxhash32, xxhash32Output[i]))
}

xxhash32 <- getVDigest(algo = 'xxhash32')
expect_identical(xxhash32(xxhash32Input, serialize = FALSE), xxhash32Output)


## these outputs were calculated using xxh64sum
## [ Correction:  These reproduce via the Python xxhash package and its hexdigest() output
##   but not the xxh64sum command-line tool as the original comment here implies. ]
xxhash64Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
xxhash64Output <-
    c("44bc2cf5ad770999",
      "f06103773e8585df",
      "ef46db3751d8e999")

for (i in seq(along.with=xxhash64Input)) {
    xxhash64 <- digest(xxhash64Input[i], algo="xxhash64", serialize=FALSE)
    #cat(xxhash64, "\n")
    expect_true(identical(xxhash64, xxhash64Output[i]))
}

xxhash64 <- getVDigest(algo = 'xxhash64')
expect_identical(xxhash64(xxhash64Input, serialize = FALSE), xxhash64Output)


## these outputs were calculated using mmh3 python package
## the first two are also shown at this StackOverflow question on test vectors
##   https://stackoverflow.com/questions/14747343/murmurhash3-test-vectors
murmur32Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
murmur32Output <-
    c("b3dd93fa",
      "ee925b90",
      "00000000")

for (i in seq(along.with=murmur32Input)) {
    murmur32 <- digest(murmur32Input[i], algo="murmur32", serialize=FALSE)
    #cat(murmur32, "\n")
    expect_true(identical(murmur32, murmur32Output[i]))
}

murmur32 <- getVDigest(algo = 'murmur32')
expect_identical(murmur32(murmur32Input, serialize = FALSE), murmur32Output)


## tests for digest spooky

## Per PR 205, see comment in https://github.com/facebook/folly/blob/4c603f8c2add8d0228de0e073c5ae3ce9b02b6f3/folly/hash/SpookyHashV2.h#L35-L36
## Values ought to be sensible on big endian too but different from little endian reference
## so we do not test on big endian
if (isTRUE(.Call(digest:::is_little_endian))) {

    ## test vectors (originally for md5)
    spookyInput <- c("",
                     "a",
                     "abc",
                     "message digest",
                     "abcdefghijklmnopqrstuvwxyz",
                     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
                     paste("12345678901234567890123456789012345678901234567890123456789012",
                           "345678901234567890", sep=""))

    # from spooky import hash128
    # from binascii import hexlify
    #
    # spookyInput = [
    #     "",
    #       "a",
    #       "abc",
    #       "message digest",
    #       "abcdefghijklmnopqrstuvwxyz",
    #       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
    #       "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
    #     ]
    #
    # for s in spookyInput:
    #     hexlify(hash128(s).to_bytes(16, 'little')).decode()
    #
    # '1909f56bfc062723c751e8b465ee728b'
    # 'bdc9bba09181101a922a4161f0584275'
    # '67c93775f715ab8ab01178caf86713c6'
    # '9630c2a55c0987a0db44434f9d67a192'
    # '5172de938ce149a98f4d06d3c3168ffe'
    # 'b5b3b2d0f08b58aa07f551895f929f81'
    # '3621ec01112dafa1610a4bd23041966b'

    spookyOutputPython <-   c('1909f56bfc062723c751e8b465ee728b',
                              'bdc9bba09181101a922a4161f0584275',
                              '67c93775f715ab8ab01178caf86713c6',
                              '9630c2a55c0987a0db44434f9d67a192',
                              '5172de938ce149a98f4d06d3c3168ffe',
                              'b5b3b2d0f08b58aa07f551895f929f81',
                              '3621ec01112dafa1610a4bd23041966b')

    ## spooky raw output test
    for (i in seq(along.with=spookyInput)) {
      # skip = 30 skips the entire serialization header for a length 1 character vector
      # this is equivalent to raw = TRUE and matches the python spooky implementation for those vectors
      spooky <- digest(spookyInput[i], algo = "spookyhash", skip = 30)
      expect_true(identical(spooky, spookyOutputPython[i]))
      #cat(spooky, "\n")
    }

    expect_identical(
        getVDigest(algo = 'spookyhash')(spookyInput, skip = 30),
        spookyOutputPython
    )

    ## some extras to get coverage up - these aren't tested against reference output,
    ## just output from R 3.6.0
    spookyInput <- c("a", "aaaaaaaaa", "aaaaaaaaaaaaa")
    spookyOutput <- c("b7a3573ba6139dfdc52db30acba87f46",
                      "fd876ecaa5d1e442600333118f223e02",
                      "91848873bf91d06ad321bbd47400a556")
    for (i in seq(along.with=spookyInput)) {
        spooky <- digest(spookyInput[i], algo = "spookyhash")
        expect_true(identical(spooky, spookyOutput[i]))
        ##cat(spooky, "\n")
    }

    expect_identical(
        getVDigest(algo = 'spookyhash')(spookyInput),
        spookyOutput
    )

    ## test a bigger object
    spooky <- digest(iris, algo = "spookyhash")
    expect_true(identical(spooky, "af58add8b4f7044582b331083bc239ff"))
    expect_identical(getVDigest('spookyhash')(list(iris)),
                     "af58add8b4f7044582b331083bc239ff")
    ##cat(spooky, "\n")

    # test error message
    #error.message <- try(digest(spookyInput[i], algo = "spookyhash", serialize = FALSE))
    #expect_true(
    #  grepl("spookyhash algorithm is not available without serialization.", error.message)
    #)
}

## Ensure that all values of algo are actually allowed (in case a new one is
## added in the future). The call to match.arg() passes choices explicitly
## because it is significantly faster to do it than to have it automatically
## infer the possible choices from the function's formals.

# Grab the possible values of algo, then call digest() for each one.
algos <- eval(formals(digest)$algo)
for (algo in algos) {
  digest(123, algo = algo)
}
# Same for getVDigest
algos <- eval(formals(getVDigest)$algo)
for (algo in algos) {
  getVDigest(algo = algo)
}


## xxhash h3_64 variant
## reference values computed via xxhash and its xxh3_64 object and hexdiges printer:
## ie print(xxhash.xxh3_64("abc").hexdigest())
xxh3_64Input <- c("abc",
                  "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
                  "")
xxh3_64Output <- c("78af5f94892f3950",
                   "5bbcbbabcdcc3d3f",
                   "2d06800538d394c2")
for (i in seq(along.with=xxh3_64Input)) {
    xxh3_64 <- digest(xxh3_64Input[i], algo="xxh3_64", serialize=FALSE)
    #cat(xxh3_64, "\n")
    expect_true(identical(xxh3_64, xxh3_64Output[i]))
}

xxh3_64 <- getVDigest(algo = 'xxh3_64')
expect_identical(xxh3_64(xxh3_64Input, serialize = FALSE), xxh3_64Output)


## xxhash h3_128 variant
## reference values computed via xxhash and its xxh3_128 object and hexdiges printer:
## ie print(xxhash.xxh3_128("abc").hexdigest())
xxh3_128Input <- c("abc",
                   "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
                   "")
xxh3_128Output <- c("06b05ab6733a618578af5f94892f3950",
                    "3d62d22a5169b016c0d894fd4828a1a7",
                    "99aa06d3014798d86001c324468d497f")
for (i in seq(along.with=xxh3_128Input)) {
    xxh3_128 <- digest(xxh3_128Input[i], algo="xxh3_128", serialize=FALSE)
    #cat(xxh3_128, "\n")
    expect_true(identical(xxh3_128, xxh3_128Output[i]))
}

xxh3_128 <- getVDigest(algo = 'xxh3_128')
expect_identical(xxh3_128(xxh3_128Input, serialize = FALSE), xxh3_128Output)

## a few tests split off the test_digest.R file

suppressMessages(library(digest))

## test 'length' parameter and file input
##fname <- file.path(R.home(),"COPYING")  ## not invariant across OSs
fname <- system.file("GPL-2", package="digest")
x <- readChar(fname, file.info(fname)$size) # read file
xskip <- substring(x, first=20+1)
for (alg in c("sha1", "md5", "crc32", "sha256", "sha512",
              "xxhash32", "xxhash64", "murmur32")) {
                                        # partial file
    h1 <- digest(x    , length=18000, algo=alg, serialize=FALSE)
    h2 <- digest(fname, length=18000, algo=alg, serialize=FALSE, file=TRUE)
    expect_true(identical(h1,h2))
    h3 <- digest(substr(x,1,18000)  , algo=alg, serialize=FALSE)
    expect_true(identical(h1,h3))
    #cat(h1, "\n", h2, "\n", h3, "\n")
    expect_identical(
      getVDigest(alg)(x, length = 18e3, serialize = FALSE),
      getVDigest(alg)(fname, length = 18e3, serialize = FALSE, file = TRUE)
    )
                                        # whole file
    h4 <- digest(x    , algo=alg, serialize=FALSE)
    h5 <- digest(fname, algo=alg, serialize=FALSE, file=TRUE)
    expect_true( identical(h4,h5) )

    expect_identical(
      getVDigest(alg)(x, serialize = FALSE),
      getVDigest(alg)(fname, serialize = FALSE, file = TRUE)
    )

    ## Assert that 'skip' works
    h6 <- digest(xskip, algo=alg, serialize=FALSE)
    h7 <- digest(fname, algo=alg, serialize=FALSE, skip=20, file=TRUE)
    expect_true( identical(h6, h7) )
    expect_identical(
      getVDigest(alg)(xskip, serialize = FALSE),
      getVDigest(alg)(fname, serialize = FALSE, skip = 20, file = TRUE)
    )
}

## compare md5 algorithm to other tools
library(tools)
##fname <- file.path(R.home(),"COPYING")  ## not invariant across OSs
fname <- system.file("GPL-2", package="digest")
h1 <- as.character(md5sum(fname))
h2 <- digest(fname, algo="md5", file=TRUE)
expect_true( identical(h1,h2) )

## Make sure we don't core dump with unreadable files.
fname <- tempfile()
#cat("Hello World, you won't have access to read me", file=fname)
on.exit(unlink(fname))
Sys.chmod(fname, mode="0000")
try(res <- digest(fname, file=TRUE), silent=TRUE)

suppressMessages(library(digest))

isWindows <- Sys.info()[["sysname"]] == "Windows"

res <- digest(charToRaw("test"), "blake3", serialize = FALSE)

# generated using the blake3 rust implementation
expected <- "4878ca0425c739fa427f7eda20fe845f6b2e46ba5fe2a14df5b1e32f50603215"
expect_equal(
  res,
  expected
)

res <- digest(charToRaw("test"), "blake3", serialize = FALSE, raw = TRUE)
expect_equal(
  paste0(res, collapse = ""),
  expected
)

## blake3 example
blake3Input <-
  c(
    "abc",
    "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
    ""
  )
blake3Output <-
  c(
    "6437b3ac38465133ffb63b75273a8db548c558465d79db03fd359c6cd5bd9d85",
    "c19012cc2aaf0dc3d8e5c45a1b79114d2df42abb2a410bf54be09e891af06ff8",
    "af1349b9f5f9a1a6a0404dea36dcc9499bcb25c9adc112b7cc9a93cae41f3262"
  )
for (i in seq(along = blake3Input)) {
  blake3 <- digest(blake3Input[i], algo = "blake3", serialize = FALSE)
  expect_equal(
    blake3,
    blake3Output[i]
  )
}

if (isWindows) exit_file("Skipping remainder on Windows platform")

# test file
# used the b3sum rust crate
file <- tempfile()
writeLines("test", file)
res <- digest(file = file, algo = "blake3")
try(file.remove(file))
expect_equal(
  res,
  "dea2b412aa90f1b43a06ca5e8b8feafec45ae1357971322749480f4e1572eaa2"
)

# try another long text
file <- tempfile()
text <- paste0(rep(LETTERS, 1000), collapse = "-")
writeLines(text, file)
res <- digest(file = file, algo = "blake3")
try(file.remove(file))
expect_equal(
  res,
  "a84aef38d9a7ad55fa458d9d1857eb895832e358385e8b6466c6af54ea46eeaa"
)

##  vdigest -- Vectorist digest functions for R
##
##  Copyright (C) 2019 - 2024  Matthew de Queljoe and Dirk Eddelbuettel
##
##  This file is part of digest.
##
##  digest is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  digest is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with digest.  If not, see <http://www.gnu.org/licenses/>.

getVDigest <- function(algo = c("md5", "sha1", "crc32", "sha256", "sha512",
                                "xxhash32", "xxhash64", "murmur32", "spookyhash",
                                "blake3", "crc32c", "xxh3_64", "xxh3_128"),
                        errormode=c("stop","warn","silent")){
    algo <- match.arg(algo, c("md5", "sha1", "crc32", "sha256", "sha512",
                              "xxhash32", "xxhash64", "murmur32", "spookyhash",
                              "blake3", "crc32c", "xxh3_64", "xxh3_128"))
    errormode <- match.arg(errormode, c("stop","warn","silent"))
    algoint <- algo_int(algo)
    non_streaming_algos <- c("md5", "sha1", "crc32", "sha256", "sha512",
                             "xxhash32", "xxhash64", "murmur32", "blake3",
                             "crc32c", "xxh3_64", "xxh3_128")
    if (algo %in% non_streaming_algos)
        return(non_streaming_digest(algo, errormode, algoint))
    streaming_digest(algo, errormode, algoint)
}

non_streaming_digest <- function(algo, errormode, algoint){
    function(object,
             serialize=TRUE,
             file=FALSE,
             length=Inf,
             skip="auto",
             ascii=FALSE,
             seed=0,
             serializeVersion=.getSerializeVersion()){

        if (is.infinite(length))
            length <- -1               # internally we use -1 for infinite len

        if (is.character(file) && missing(object)) {
            object <- file                  # nocov
            file <- TRUE                  	# nocov
        }

        if (serialize && !file) {
            ## support the 'nosharing' option in pqR's serialize()
            object <- if (.hasNoSharing())
                serialize_(
                    object,
                    connection = NULL,
                    ascii = ascii,
                    nosharing = TRUE,
                    version = serializeVersion
                )
            else
                serialize_(object,
                           connection = NULL,
                           ascii = ascii,
                           version = serializeVersion)
            ## we support raw vectors, so no mangling of 'object' is necessary
            ## regardless of R version
            ## skip="auto" - skips the serialization header [SU]
            if (any(!is.na(pmatch(skip,"auto"))))
                skip <- set_skip(object, ascii)

        } else if (!is.character(object) && !inherits(object,"raw")) {
            return(.errorhandler(paste("Argument object must be of type character",	  # #nocov
                                       "or raw vector if serialize is FALSE"), mode=errormode)) # #nocov
        }
        if (file && !is.character(object))
            return(.errorhandler("file=TRUE can only be used with a character object",          # #nocov
                                 mode=errormode))                                               # #nocov

        if (file) {
            algoint <- algoint+100
            object <- path.expand(object)
            if (.isWindows()) object <- enc2utf8(object)
            check_file(object, errormode)
        }
        ## if skip is auto (or any other text for that matter), we just turn it
        ## into 0 because auto should have been converted into a number earlier
        ## if it was valid [SU]
        if (is.character(skip)) skip <- 0
        val <- .Call(
            vdigest_impl,
            object,
            as.integer(algoint),
            as.integer(length),
            as.integer(skip),
            0L, # raw always FALSE
            as.integer(seed)
        )
        ## crc32 output was not guaranteed to be eight chars long, which we corrected
        ## this allows to get the old behaviour back for compatibility
        if ((algoint == 3 || algoint == 103) && .getCRC32PreferOldOutput()) {
            val <- sub("^0+", "", val)                                          		# #nocov
        }

        return(val)
    }
}

streaming_digest <- function(algo, errormode, algoint){
    function(object,
             serialize=TRUE,
             file=FALSE,
             length=Inf,
             skip="auto",
             ascii=FALSE,
             seed=0,
             serializeVersion=.getSerializeVersion()){

        if (is.infinite(length))
            length <- -1               # internally we use -1 for infinite len

        if (is.character(file) && missing(object)) {
            object <- file                  # nocov
            file <- TRUE                  	# nocov
        }

        if (!serialize){
            .errorhandler(paste0(algo, " algorithm is not available without serialization."),  # #nocov
                          mode=errormode)                                                      # #nocov
        }

        if (serialize && !file) {
            ## we support raw vectors, so no mangling of 'object' is necessary
            ## regardless of R version
            ## skip="auto" - skips the serialization header [SU]
            if (any(!is.na(pmatch(skip,"auto"))))
                skip <- set_skip(object, ascii)
        }

        if (file)
            return(.errorhandler(paste0(algo, " algorithm can not be used with files."),        # #nocov
                                 mode=errormode))                                               # #nocov


        ## if skip is auto (or any other text for that matter), we just turn it
        ## into 0 because auto should have been converted into a number earlier
        ## if it was valid [SU]
        if (is.character(skip)) skip <- 0                                          		# #nocov
        if (algo == "spookyhash"){
            # 0s are the seeds. They are included to enable testing against fastdigest.
            val <- vapply(object,
                          function(o)
                              paste(
                                  .Call(spookydigest_impl, o, skip, 0, 0, serializeVersion),
                                  collapse = ""
                              ),
                          character(1),
                          USE.NAMES = FALSE)
        }

        ## crc32 output was not guaranteed to be eight chars long, which we corrected
        ## this allows to get the old behaviour back for compatibility
        if ((algoint == 3 || algoint == 103) && .getCRC32PreferOldOutput()) {
            val <- sub("^0+", "", val)                                          		# #nocov
        }

        return(val)
    }
}

serialize_ <- function(object, ...){
    if (length(object))
        return(lapply(object, serialize, ...))
    serialize(object, ...)
}

##  sha1 -- SHA1 hash generation for R
##
##  Copyright (C) 2015 - 2019  Thierry Onkelinx and Dirk Eddelbuettel
##  Copyright (C) 2016 - 2019  Viliam Simko
##
##  This file is part of digest.
##
##  digest is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  digest is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with digest.  If not, see <http://www.gnu.org/licenses/>.


# functions written by Thierry Onkelinx
sha1 <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    UseMethod("sha1")
}

sha1.default <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    if (is.list(x)) {
        return(
            sha1.list(x, digits = digits, zapsmall = zapsmall, ..., algo = algo)
        )
    }
    stop(  							# #nocov start
        "sha1() has no method for the '",
        paste(class(x), collapse = "', '"),
        "' class",
        call. = FALSE
    )								# #nocov end
}

sha1.numeric <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    y <- num2hex(
        x,
        digits = digits,
        zapsmall = zapsmall
    )
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}

sha1.matrix <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    # needed to make results comparable between 32-bit and 64-bit
    if (storage.mode(x) == "double") {
        y <- matrix( #return a matrix with the same dimensions as x
            apply(
                x,
                2,
                num2hex,
                digits = digits,
                zapsmall = zapsmall
            ),
            ncol = ncol(x)
        )
        y <- add_attributes(x, y)
        attr(y, "digest::sha1") <- attr_sha1(
            x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
        )
        digest(y, algo = algo)
    } else {
        attr(x, "digest::sha1") <- attr_sha1(
            x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
        )
        digest(x, algo = algo)
    }
}

sha1.complex <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    # a vector of complex numbers is converted into 2-column matrix (Re,Im)
    y <- cbind(Re(x), Im(x))
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    sha1(y, digits = digits, zapsmall = zapsmall, ..., algo = algo)
}

sha1.Date <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    y <- as.numeric(x)
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    sha1(y, digits = digits, zapsmall = zapsmall, ..., algo = algo)
}

sha1.array <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    # Array x encoded as list of two elements:
    # 1. lengths of all dimensions of x
    # 2. all cells of x as a single vector
    y <- list(
        dimension = dim(x),
        value = as.numeric(x))
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    sha1(y, digits = digits, zapsmall = zapsmall, ..., algo = algo)
}

sha1.data.frame <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    if (length(x)) {
        # needed to make results comparable between 32-bit and 64-bit
        y <- vapply(
            x,
            sha1,
            digits = digits,
            zapsmall = zapsmall,
            ...,
            algo = algo,
            FUN.VALUE = NA_character_
        )
    } else {
        y <- x
    }
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}

sha1.list <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    if (length(x)) {
        # needed to make results comparable between 32-bit and 64-bit
        y <- vapply(
            x,
            sha1,
            digits = digits,
            zapsmall = zapsmall,
            ...,
            algo = algo,
            FUN.VALUE = NA_character_
        )
    } else {
        y <- x
    }
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- list(
        class = class(x),
        digits = as.integer(digits),
        zapsmall = as.integer(zapsmall),
        ... = ...
    )
    digest(y, algo = algo)
}

sha1.POSIXlt <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    y <- do.call(
        data.frame,
        lapply(unclass(as.POSIXlt(x)), unlist)
    )
    y$sec <- num2hex(y$sec, digits = digits, zapsmall = zapsmall)
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}

sha1.POSIXct <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    y <- sha1(
        as.POSIXlt(x),
        digits = digits,
        zapsmall = zapsmall,
        ...,
        algo = algo
    )
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}

sha1.anova <- function(x, digits = 4L, zapsmall = 7L, ..., algo = "sha1"){
    if (digits > 4) {
        warning("Hash on 32 bit might be different from hash on 64 bit with digits > 4") # #nocov
    }
    y <- apply(
        x,
        1,
        num2hex,
        digits = digits,
        zapsmall = zapsmall
    )
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}

sha1.pairlist <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    # needed to make results comparable between 32-bit and 64-bit
    y <- vapply(
        x,
        sha1,
        digits = digits,
        zapsmall = zapsmall,
        ...,
        algo = algo,
        FUN.VALUE = NA_character_
    )
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}

sha1.function <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    dots <- list(...)
    if (is.null(dots$environment)) {
        dots$environment <- TRUE
    }
    if (isTRUE(dots$environment)) {
        y <- list(
            formals = formals(x),
            body = as.character(body(x)),
            environment = digest(environment(x), algo = algo)
        )
    } else {
        y <- list(
            formals = formals(x),
            body = as.character(body(x))
        )
    }
    y <- vapply(
        y,
        sha1,
        digits = digits,
        zapsmall = zapsmall,
        environment = dots$environment,
        ... = dots,
        algo = algo,
        FUN.VALUE = NA_character_
    )
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = y, digits = digits, zapsmall = zapsmall, algo = algo, dots
    )
    digest(y, algo = algo)
}

sha1.formula <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1"){
    dots <- list(...)
    if (is.null(dots$environment)) {
        dots$environment <- TRUE
    }
    y <- vapply(
        x,
        sha1,
        digits = digits,
        zapsmall = zapsmall,
        ... = dots,
        algo = algo,
        FUN.VALUE = NA_character_
    )
    if (isTRUE(dots$environment)) {
        y <- c(
            y,
            digest(environment(x), algo = algo)
        )
    }
    y <- add_attributes(x, y)
    attr(y, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(y, algo = algo)
}
"sha1.(" <- function(...) {sha1.formula(...)}

add_attributes <- function(x, y) {
    if (.getsha1PackageVersion() < package_version("0.6.22.2")) {
        return(y)
    }
    extra <- attributes(x)
    if (package_version("0.6.23.2") <= .getsha1PackageVersion()) {
        extra <- extra[names(extra) != "srcref"]
    }
    attributes(y) <- c(attributes(y), "digest::attributes" = extra)
    return(y)
}

# sha1_attr_digest variants ####

sha1_attr_digest <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    # attributes can be set on objects calling sha1_attr_digest()
    attr(x, "digest::sha1") <- attr_sha1(
        x = x, digits = digits, zapsmall = zapsmall, algo = algo, ...
    )
    digest(x, algo = algo)
}

sha1.call <- function(...) {sha1_attr_digest(...)}
sha1.character <- function(...) {sha1_attr_digest(...)}
sha1.factor <- function(...) {sha1_attr_digest(...)}
sha1.logical <- function(...) {sha1_attr_digest(...)}
sha1.integer <- function(...) {sha1_attr_digest(...)}
sha1.raw <- function(...) {sha1_attr_digest(...)}

# sha1_digest variants ####

sha1_digest <- function(x, digits = 14L, zapsmall = 7L, ..., algo = "sha1") {
    # attributes cannot be set on objects calling sha1_digest()
    digest(x, algo = algo)
}

sha1.name <- function(...) {sha1_digest(...)}
sha1.NULL <- function(...) {sha1_digest(...)}

# Support Functions ####

attr_sha1 <- function(x, digits, zapsmall, algo, ...) {
    if (algo == "sha1") {
        return(
            list(
                class = class(x),
                digits = as.integer(digits),
                zapsmall = as.integer(zapsmall),
                ...
            )
        )
    }
    list(
        class = class(x),
        digits = as.integer(digits),
        zapsmall = as.integer(zapsmall),
        algo = algo,
        ...
    )
}

num2hex <- function(x, digits = 14L, zapsmall = 7L){
    if (!is.numeric(x)) {
        stop("x is not numeric")				# #nocov
    }
    if (!is.integer(digits)) {
        if (!all.equal(as.integer(digits), digits)) {
            stop("digits is not integer")			# #nocov
        }
        digits <- as.integer(digits)
    }
    if (length(digits) != 1) {
        stop("digits must contain exactly one integer")		# #nocov
    }
    if (digits < 1) {
        stop("digits must be positive")				# #nocov
    }
    if (!is.integer(zapsmall)) {
        if (!all.equal(as.integer(zapsmall), zapsmall)) {
            stop("zapsmall is not integer")			# #nocov
        }
        zapsmall <- as.integer(zapsmall)
    }
    if (length(zapsmall) != 1) {
        stop("zapsmall must contain exactly one integer")	# #nocov
    }
    if (zapsmall < 1) {
        stop("zapsmall must be positive")			# #nocov
    }
    if (length(x) == 0) {
        return(character(0))
    }
    x.na <- is.na(x)
    if (all(x.na)) {
        return(x)
    }
    x.inf <- is.infinite(x)
    output <- rep(NA_character_, length(x))
    output[x.inf & x > 0] <- "Inf"
    output[x.inf & x < 0] <- "-Inf"
    # detect "small" numbers
    x.zero <- !x.na & !x.inf & abs(x) <= (2^floor(log2(10 ^ -zapsmall)))
    output[x.zero] <- "0"
    # The calculations for non-na, non-inf, non-zero values are computationally
    # more intense.  Don't do them unless necessary.
    x.finite <- !(x.na | x.inf | x.zero)
    if (!any(x.finite)) {
        return(output)
    }
    x_abs <- abs(x[x.finite])
    exponent <- floor(log2(x_abs))
    negative <- c("", "-")[(x[x.finite] < 0) + 1]
    x.hex <- sprintf("%a", x_abs*2^-exponent)
    nc_x <- nchar(x.hex)
    digits.hex <- ceiling(log(10 ^ digits, base = 16))
    # select mantissa (starting format is 0x1.[0-9a-f]+p[+-][0-9]+), remove the
    # beginning through the decimal place, including the fact that exact powers
    # of two will not have a decimal place.
    # Remove the beginning through the decimal place (if it exists).
    mask_decimal <- startsWith(x.hex, "0x1.")
    start_character <- 4 + mask_decimal
    # select required precision
    stop_character <- pmin(nc_x - 3, start_character + digits.hex - 1)
    mantissa <- substring(x.hex, start_character, stop_character)
    # Drop trailing zeros
    mantissa <- gsub(x = mantissa, pattern = "0*$", replacement = "")
    output[x.finite] <- sprintf("%s%s %d", negative, mantissa, exponent)
    return(output)
}

## tests for digest, taken from the examples in the manual page

suppressMessages(library(digest))

# calculate sha1 fingerprints
x.numeric <- c(seq(0, 1, length = 4 ^ 3), -Inf, Inf, NA, NaN)
x.list <- list(letters, x.numeric)
x.dataframe <- data.frame(X = letters,
                          Y = x.numeric[2],
                          Z = factor(letters),
                          stringsAsFactors = FALSE)
x.matrix.num <- as.matrix(x.numeric)
x.matrix.letter <- as.matrix(letters)
x.dataframe.round <- x.dataframe
x.dataframe.round$Y <- signif(x.dataframe.round$Y, 14)
x.factor <- factor(letters)
x.array.num <- as.array(x.numeric)
x.formula <- a~b+c|d
x.paren_formula <- a~(b+c)
x.no_paren_formula <- a~b+c

# tests using detailed numbers
expect_false(identical(x.numeric, signif(x.numeric, 14)))
expect_false(identical(x.matrix.num, signif(x.matrix.num, 14)))

# returns the correct SHA1
expect_true(
    identical(
        sha1(x.numeric),
        {
            z <- digest:::num2hex(x.numeric)
            attr(z, "digest::sha1") <- list(
                class = class(x.numeric),
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)
# Verify that all numeric values (especially +-Inf and NA/NaN) return unique
# SHA1 hashes
expect_false(
    any(duplicated(sapply(x.numeric, sha1)))
)
expect_true(
    identical(
        sha1(letters),
        {
            z <- letters
            attr(z, "digest::sha1") <- list(
                class = "character",
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)
expect_true(
    identical(
        sha1(x.list),
        {
            z <- sapply(x.list, sha1)
            attr(z, "digest::sha1") <- list(
                class = "list",
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)
options(sha1PackageVersion = "0.6.22.1")
expect_true(
    identical(
        sha1(x.dataframe),
        {
            z <- sapply(x.dataframe, sha1)
            attr(z, "digest::sha1") <- list(
                class = "data.frame",
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)
## expect_true(
##     identical(
##         sha1(x.matrix.num),
##         {
##             z <- matrix(
##                 apply(x.matrix.num, 2, digest:::num2hex),
##                 ncol = ncol(x.matrix.num)
##             )
##             attr(z, "digest::sha1") <- list(
##                 class = "matrix",
##                 digits = 14L,
##                 zapsmall = 7L
##             )
##             digest(z, algo = "sha1")
##         }
##     )
## )
## expect_true(
##     identical(
##         sha1(x.matrix.letter),
##         {
##             z <- x.matrix.letter
##             attr(z, "digest::sha1") <- list(
##                 class = "matrix",
##                 digits = 14L,
##                 zapsmall = 7L
##             )
##             digest(z, algo = "sha1")
##         }
##     )
## )
stopifnot(
    identical(
        sha1(x.factor),
        {
            z <- x.factor
            attr(z, "digest::sha1") <- list(
                class = "factor",
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)
# a matrix and a vector should have a different hash
expect_true(
    !identical(
        sha1(x.numeric),
        sha1(matrix(x.numeric, nrow = 1))
    )
)
expect_true(
    !identical(
        sha1(x.numeric),
        sha1(matrix(x.numeric, ncol = 1))
    )
)
expect_true(
    !identical(
        sha1(letters),
        sha1(matrix(letters, nrow = 1))
    )
)
expect_true(
    !identical(
        sha1(letters),
        sha1(matrix(letters, ncol = 1))
    )
)

# character(0) and numeric(0) should have a different hash
expect_true(!identical(sha1(character(0)), sha1(numeric(0))))

# a POSIXct and a POSIXlt should give a different hash
z <- as.POSIXct("2015-01-02 03:04:06.07", tz = "UTC")
expect_true(
    !identical(
        sha1(z),
        sha1(as.POSIXlt(z))
    )
)

lm.model.0 <- lm(weight ~ Time, data = ChickWeight)
lm.model.1 <- lm(weight ~ 1, data = ChickWeight)
glm.model.0 <- glm(weight ~ Time, data = ChickWeight, family = poisson)
glm.model.1 <- glm(weight ~ 1, data = ChickWeight, family = poisson)

anova.list <- list(
    lm = anova(lm.model.0, lm.model.1),
    glm = anova(glm.model.0, glm.model.1),
    glm.test = anova(glm.model.0, glm.model.1, test = "Chisq")
)

# works with lm anova"
expect_true(
    identical(
        sha1(anova.list[["lm"]]),
        {
            y <- apply(
                anova.list[["lm"]],
                1,
                digest:::num2hex,
                digits = 4,
                zapsmall = 7
            )
            attr(y, "digest::sha1") <- list(
                class = c("anova", "data.frame"),
                digits = 4L,
                zapsmall = 7L
            )
            digest(y, algo = "sha1")
        }
    )
)
# works with glm anova"
expect_true(
    identical(
        sha1(anova.list[["glm"]]),
        {
            y <- apply(
                anova.list[["glm"]],
                1,
                digest:::num2hex,
                digits = 4,
                zapsmall = 7
            )
            attr(y, "digest::sha1") <- list(
                class = c("anova", "data.frame"),
                digits = 4L,
                zapsmall = 7L
            )
            digest(y, algo = "sha1")
        }
    )
)
expect_true(
    identical(
        sha1(anova.list[["glm.test"]]),
        {
            y <- apply(
                anova.list[["glm.test"]],
                1,
                digest:::num2hex,
                digits = 4,
                zapsmall = 7
            )
            attr(y, "digest::sha1") <- list(
                class = c("anova", "data.frame"),
                digits = 4L,
                zapsmall = 7L
            )
            digest(y, algo = "sha1")
        }
    )
)
expect_true(
    identical(
        sha1(x.formula, environment=FALSE),
        {
            y <- sapply(
                X=x.formula,
                FUN=sha1,
                digits=14L,
                zapsmall=7L,
                ...=list(environment=FALSE),
                algo="sha1"
            )
            attr(y, "digest::sha1") <- list(
                class="formula",
                digits=14L,
                zapsmall=7L,
                environment=FALSE
            )
            digest(y, algo="sha1")
        }
    )
)
expect_true(
    identical(
        sha1(x.formula),
        {
            y <- c(
                sapply(
                    X=x.formula,
                    FUN=sha1,
                    digits=14L,
                    zapsmall=7L,
                    ...=list(environment=TRUE),
                    algo="sha1"
                ),
                digest(environment(x.formula), algo="sha1")
            )
            attr(y, "digest::sha1") <- list(
                class="formula",
                digits=14L,
                zapsmall=7L
            )
            digest(y, algo="sha1")
        }
    )
)
expect_true(
    sha1(x.paren_formula) != sha1(x.no_paren_formula)
)

test.element <- list(
    # NULL
    NULL,
    # empty classes
    logical(0), integer(0), numeric(0), character(0), list(), data.frame(),
    # scalar
    TRUE, FALSE, 1L, 1, "a",
    # date. Make sure to add the time zone. Otherwise the test might fail
    as.POSIXct("2015-01-02 03:04:06.07", tz = "UTC"),
    # vector
    c(TRUE, FALSE), 1:3, seq(0, 10, length = 4), letters[1:3],
    factor(letters[4:6]),
    as.POSIXct(c("2015-01-02 03:04:06.07", "1960-12-31 23:59:59"), tz = "UTC")
)
select.vector <- which(sapply(test.element, length) > 1)
test.element <- c(
    test.element,
    # add a data.frame
    list(expand.grid(test.element[select.vector])),
    # add a list
    list(test.element[select.vector]),
    # add matrices
    list(matrix(1:10)),
    list(matrix(seq(0, 10, length = 4))),
    list(matrix(letters))
)
# different values for digits or zapsmall gives different hashes
# expect for NULL
expect_true(
    identical(
        sha1(NULL, digits = 14),
        sha1(NULL, digits = 13)
    )
)
expect_true(
    identical(
        sha1(NULL, zapsmall = 14),
        sha1(NULL, zapsmall = 13)
    )
)
for (i in tail(seq_along(test.element), -1)) {
    expect_true(
        !identical(
            sha1(test.element[[i]], digits = 14),
            sha1(test.element[[i]], digits = 13)
        )
    )
    expect_true(
        !identical(
            sha1(test.element[[i]], zapsmall = 7),
            sha1(test.element[[i]], zapsmall = 6)
        )
    )
}
test.element <- c(test.element, anova.list)

#cat("\ncorrect <- c(\n")
#cat(
#    sprintf("    \"%s\"", sapply(test.element, sha1)),
#    sep = ",\n"
#)
#cat(")\n")

correct <- c(
    "8d9c05ec7ae28b219c4c56edbce6a721bd68af82",
    "d61eeea290dd09c5a3eba41c2b3174b6e4e2366d",
    "af23305d27f0409c91bdb86ba7c0cdb2e09a5dc6",
    "0c9ca70ce773deb0d9c0b0579c3b94856edf15cc",
    "095886422ad26e315c0960ef6b09842a1f9cc0ce",
    "6cc04c6c432bb91e210efe0b25c6ca809e6df2e3",
    "c1113ba008a349de64da2a7a724e501c1eb3929b",
    "6e12370bdc6fc457cc154f0525e22c6aa6439f4d",
    "1c1b5393c68a643bc79c277c6d7374d0b30cd985",
    "b48c17a2ac82601ff38df374f87d76005fb61cbd",
    "35280c99aa6a48bfc2810b72b763ccac0f632207",
    "f757cc017308d217f35ed8f0c001a57b97308fb7",
    "cfcf101b8449af67d34cdc1bcb0432fe9e4de08e",
    "a14384d1997440bad13b97b3ccfb3b8c0392e79a",
    "555f6bea49e58a2c2541060a21c2d4f9078c3086",
    "631d18dec342e2cb87614864ba525ebb9ad6a124",
    "b6c04f16b6fdacc794ea75c8c8dd210f99fafa65",
    "25485ba7e315956267b3fdc521b421bbb046325d",
    "6def3ca353dfc1a904bddd00e6a410d41ac7ab01",
    "cf220bcf84c3d0ab1b01f8f764396941d15ff20f",
    "2af8021b838f613aee7670bed19d0ddf1d6bc0c1",
    "270ed85d46524a59e3274d89a1bbf693521cb6af",
    "60e09482f12fda20f7d4a70e379c969c5a73f512",
    "10380001af2a541b5feefc7aab9f719b67330a42",
    "4580ff07f27eb8321421efac1676a80d9239572a",
    "d3022c5a223caaf77e9c564e024199e5d6f51bd5",
    "f54742ac61edd8c3980354620816c762b524dfc7"
)
# each object should yield a different hash
expect_true(!any(duplicated(correct)))
# returns the same SHA1 on both 32-bit and 64-bit OS"
## for (i in seq_along(test.element)) {
##     expect_true(
##         identical(
##             sha1(test.element[[i]]),
##             correct[i]
##         )
##     )
## }

# does work with empty lists and data.frames
expect_true(is.character(sha1(list())))
expect_true(is.character(sha1(data.frame())))
expect_true(is.character(sha1(list(a = 1, b = list(), c = data.frame()))))

# does work with complex type
expect_true(is.character(sha1(2 + 5i))) # single complex number
expect_true(is.character(sha1(1:10 + 5i))) # vector of complex numbers

# complex number with only the real part should be different from real number
expect_true(sha1(2) != sha1(2 + 0i))

# does work with Date type
expect_true(is.character(sha1(Sys.Date())))
expect_true(sha1(as.Date("1980-01-01")) != sha1(as.Date("1990-01-01")))

# different hashes for differently shaped arrays that contain the same data
data <- 1:8
a <- array(data, dim = c(2,2,2)) # cube 2x2x2
b <- array(data, dim = c(2,4,1)) # matrix 2x4
expect_true(sha1(a) != sha1(b))

# test error message
junk <- pi
class(junk) <- c("A", "B")
#error.message <- try(sha1(junk))
#expect_true(grepl("sha1\\(\\) has no method for the 'A', 'B' class", error.message))

junk <- function(
    x, y = list(...), test = TRUE, start = NULL, text = "abc", family = poisson,
    ...
){
    sha1(x)
}
#expect_true(sha1(junk) == "be194e8cdae926c13fd4e2c65bf6cb7a28dd0505")
expect_true(sha1(junk) == sha1(junk, environment = TRUE))
expect_true(sha1(junk) != sha1(junk, environment = FALSE))

#expect_true(sha1(matrix(integer(0))) == "e13485e1b995f3e36d43674dcbfedea08ce237bc")
expect_true(
    !identical(
        sha1(matrix(integer(0))),
        sha1(matrix(character(0)))
    )
)
expect_true(
    !identical(
        sha1(matrix(integer(0))),
        sha1(matrix(numeric(0)))
    )
)

## if (getRversion() < "3.5.0") {
##     expect_true(
##         identical(
##             sha1(serialize("e13485e1b995f3e36d43674dcbfedea08ce237bc", NULL)),
##             "93ab6a61f1a2ad50d4bf58396dc38cd3821b2eaf"
##         )
##     )
## }

x <- letters
for (algo in c("md5", "sha1", "crc32", "sha256", "sha512", "xxhash32",
               "xxhash64",  "murmur32")) {
    y <- x
    attr(y, "digest::sha1") <- digest:::attr_sha1(x, 14L, 7L, algo = algo)
    expect_true(
        identical(
            sha1(x, algo = algo),
            digest(y, algo = algo)
        )
    )

}

expect_true(is.character(sha1(sessionInfo())))

# check the effect of attributes from version 0.6.22.2
options(sha1PackageVersion = utils::packageVersion("digest"))
check_attribute_effect <- function(x) {
    y <- x
    attr(y, "test") <- "junk"
    expect_false(sha1(x) == sha1(y))
}
test.element <- list(2 + 5i, x.array.num, Sys.Date(), Sys.time(), y ~ x)
test.element <- c(test.element, list(x.dataframe), anova.list, function(x){x})
for (z in test.element) {
    check_attribute_effect(z)
}

# check that sha1() on contributed functions maintain there hash after storing
f <- tempfile(fileext = ".rds")
x <- digest::sha1
saveRDS(x, f)
y <- readRDS(f)
expect_identical(sha1(x), sha1(y))
expect_identical(sha1(x, environment = FALSE), sha1(y, environment = FALSE))

## tests for raw output

suppressMessages(library(digest))


current <- hmac("Jefe", 'what do ya want for nothing?', "md5", raw=TRUE)
expected <- as.raw(c(0x75, 0x0c, 0x78, 0x3e, 0x6a, 0xb0, 0xb5, 0x03, 0xea,
                     0xa8, 0x6e, 0x31, 0x0a, 0x5d, 0xb7, 0x38))
expect_equal(current, expected)


current <- digest("The quick brown fox", algo="sha1", raw=TRUE)
expected <- as.raw(c(0x5f, 0x79, 0x8c, 0xb4, 0xd8, 0x14, 0x4e, 0xec, 0x35,
                     0xf4, 0xd0, 0x79, 0x3e, 0xf2, 0x1e, 0x55, 0xce, 0xb6, 0xa7, 0x88))
expect_equal(current, expected)


## feed raw to sha1() to test sha1.raw() as well
expect_true(sha1(expected) == "75a2995eeec0fcb5d7fa97c676a37f4e224981a1")

## tests for digest, taken from the examples in the manual page

suppressMessages(library(digest))


# zap small numbers to zero
zapsmall <- 1:10
border <- 2 ^ floor(log2(10 ^ -zapsmall))
expect_equal(
    sapply(
        seq_along(zapsmall),
        function(i) { digest:::num2hex(border[i] * -1:1, digits = 6, zapsmall = zapsmall[i]) }
    ),
    matrix(
        "0",
        ncol = length(zapsmall),
        nrow = 3
    )
)

# handle 0 correct
expect_equal(digest:::num2hex(0), "0")


# digits are consistent
x <- pi
x.hex <- sapply(1:16, digest:::num2hex, x = x)
x.hex <- x.hex[c(TRUE, diff(nchar(x.hex)) > 0)]
exponent <-  unique(gsub("^[0-9a-f]* ", "", x.hex))
expect_equal(length(exponent), 1L)

mantissa <- gsub(" [0-9]*$", "", x.hex)
ignore(expect_true)(all(
    sapply(
        head(seq_along(mantissa), -1),
        function(i){
            all(
                grepl(
                    paste0("^", mantissa[i], ".*"),
                    tail(mantissa, -i)
                )
            )
        }
    )
))

#it keeps NA values
x <- c(pi, NA, 0)
expect_equal(is.na(digest:::num2hex(x)), is.na(x))

x <- c(pi, NA, pi)
expect_equal(is.na(digest:::num2hex(x)), is.na(x))

x <- as.numeric(c(NA, NA, NA))
expect_equal(is.na(digest:::num2hex(x)), is.na(x))


# handles empty vectors
expect_equal(digest:::num2hex(numeric(0)), character(0))


# example from FAQ 7.31
# tests if all trailing zero's are removed
expect_true(identical(digest:::num2hex(2, digits = 14),
                    digest:::num2hex(sqrt(2) ^ 2, digits = 14)))

expect_true(!identical(digest:::num2hex(2, digits = 15),
                     digest:::num2hex(sqrt(2) ^ 2, digits = 15)))

if (getRversion() >= '4.0.0') exit_file("Skip tests for R 4.0.0 or later")

library(digest)

x.numeric <- c(seq(0, 1, length = 4 ^ 3), -Inf, Inf, NA, NaN)
x.list <- list(letters, x.numeric)
x.dataframe <- data.frame(X = letters,
                          Y = x.numeric[2],
                          Z = factor(letters),
                          stringsAsFactors = FALSE)
x.matrix.num <- as.matrix(x.numeric)
x.matrix.letter <- as.matrix(letters)
x.dataframe.round <- x.dataframe
x.dataframe.round$Y <- signif(x.dataframe.round$Y, 14)
x.factor <- factor(letters)
x.array.num <- as.array(x.numeric)
x.formula <- a~b+c|d
x.paren_formula <- a~(b+c)
x.no_paren_formula <- a~b+c

test.element <- list(
    # NULL
    NULL,
    # empty classes
    logical(0), integer(0), numeric(0), character(0), list(), data.frame(),
    # scalar
    TRUE, FALSE, 1L, 1, "a",
    # date. Make sure to add the time zone. Otherwise the test might fail
    as.POSIXct("2015-01-02 03:04:06.07", tz = "UTC"),
    # vector
    c(TRUE, FALSE), 1:3, seq(0, 10, length = 4), letters[1:3],
    factor(letters[4:6]),
    as.POSIXct(c("2015-01-02 03:04:06.07", "1960-12-31 23:59:59"), tz = "UTC")
)

expect_true(
    identical(
        sha1(x.matrix.num),
        {
            z <- matrix(
                apply(x.matrix.num, 2, digest:::num2hex),
                ncol = ncol(x.matrix.num)
            )
            z <- digest:::add_attributes(x.matrix.num, z)
            attr(z, "digest::sha1") <- list(
                class = "matrix",
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)

expect_true(
    identical(
        sha1(x.matrix.letter),
        {
            z <- x.matrix.letter
            attr(z, "digest::sha1") <- list(
                class = "matrix",
                digits = 14L,
                zapsmall = 7L
            )
            digest(z, algo = "sha1")
        }
    )
)

correct <- c(
    "8d9c05ec7ae28b219c4c56edbce6a721bd68af82",
    "d61eeea290dd09c5a3eba41c2b3174b6e4e2366d",
    "af23305d27f0409c91bdb86ba7c0cdb2e09a5dc6",
    "0c9ca70ce773deb0d9c0b0579c3b94856edf15cc",
    "095886422ad26e315c0960ef6b09842a1f9cc0ce",
    "6cc04c6c432bb91e210efe0b25c6ca809e6df2e3",
    "c1113ba008a349de64da2a7a724e501c1eb3929b",
    "6e12370bdc6fc457cc154f0525e22c6aa6439f4d",
    "1c1b5393c68a643bc79c277c6d7374d0b30cd985",
    "b48c17a2ac82601ff38df374f87d76005fb61cbd",
    "35280c99aa6a48bfc2810b72b763ccac0f632207",
    "f757cc017308d217f35ed8f0c001a57b97308fb7",
    "cfcf101b8449af67d34cdc1bcb0432fe9e4de08e",
    "a14384d1997440bad13b97b3ccfb3b8c0392e79a",
    "555f6bea49e58a2c2541060a21c2d4f9078c3086",
    "631d18dec342e2cb87614864ba525ebb9ad6a124",
    "b6c04f16b6fdacc794ea75c8c8dd210f99fafa65",
    "25485ba7e315956267b3fdc521b421bbb046325d",
    "6def3ca353dfc1a904bddd00e6a410d41ac7ab01",
    "cf220bcf84c3d0ab1b01f8f764396941d15ff20f",
    "2af8021b838f613aee7670bed19d0ddf1d6bc0c1",
    "270ed85d46524a59e3274d89a1bbf693521cb6af",
    "60e09482f12fda20f7d4a70e379c969c5a73f512",
    "10380001af2a541b5feefc7aab9f719b67330a42",
    "4580ff07f27eb8321421efac1676a80d9239572a",
    "d3022c5a223caaf77e9c564e024199e5d6f51bd5",
    "f54742ac61edd8c3980354620816c762b524dfc7"
)

for (i in seq_along(test.element)) {
    if (i == 7 || i == 13 || i == 19) next
    expect_true(
        identical(
            sha1(test.element[[i]]),
            correct[i]
        )
    )
}

expect_true(
    sha1(matrix(integer(0))) == "e13485e1b995f3e36d43674dcbfedea08ce237bc"
)

##  hmac -- Hash-based message authentication code for R
##
##  Copyright (C) 2011 - 2019  Mario Frasca and Dirk Eddelbuettel
##  Copyright (C) 2012 - 2019  Hannes Muehleisen and Dirk Eddelbuettel
##
##  This file is part of digest.
##
##  digest is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  digest is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with digest.  If not, see <http://www.gnu.org/licenses/>.

makeRaw <- function(object)
    ## generic function, converts an object to a raw
    UseMethod('makeRaw')

makeRaw.raw <- function(object) object

makeRaw.character <- function(object) charToRaw(object)

# splits a hex-string into the values it contains.
makeRaw.digest <- function(object) {
    parts <- sapply(seq(1, nchar(object), 2),
                    function(i) { substr(object, i, i + 1) })
    as.raw(as.hexmode(parts))
}

makeRaw.default <- function(object) as.raw(object)

# key shall be padded if its length is smaller than the block size of the
# respective algorithm, hashed if longer. block sizes are as follows:
#
# md5		64
# sha1		64
# crc32	 	4
# sha256 	64
# sha512	128 (doh!)

padWithZeros <- function(k,algo) {
    blocksize <- 64
    if (algo == "crc32") blocksize <- 4			  # #nocov
    if (algo == "sha512") blocksize <- 128
    k <- makeRaw(k)
    if(length(k) > blocksize) {# not while()
        k <-digest(k, algo=algo, serialize=FALSE,raw=TRUE)
        if (algo == "crc32") {
            k <- substring(k, seq(1,7,2), seq(2,8,2))  	  # #nocov
            k <- makeRaw(strtoi(k,16))                    # #nocov
        }
    }
    makeRaw(c(k, rep(0, blocksize - length(k))))
}

hmac <- function(key, object,
                 algo=c("md5", "sha1", "crc32", "sha256", "sha512"),
                 serialize=FALSE, raw=FALSE, ...) {
    padded.key <- padWithZeros(key,algo)
    i.xored.key <- xor(padded.key, makeRaw(0x36))
    character.digest <- digest(c(i.xored.key, makeRaw(object)),
                               algo=algo, serialize=serialize, ...)
    raw.digest <- makeRaw.digest(character.digest)
    o.xored.key <- xor(padded.key, makeRaw(0x5c))
    result <- digest(c(o.xored.key, raw.digest), algo=algo, serialize=serialize, ...)
    if (raw) result <- makeRaw.digest(result)
    return(result)
}

## tests for hmac, taken from the examples in the manual page

suppressMessages(library(digest))

## Standard RFC 2104 test vectors
current <- hmac('Jefe', 'what do ya want for nothing?', "md5")
target <- '750c783e6ab0b503eaa86e310a5db738'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

current <- hmac('Jefe', 'what do ya want for nothing?', "md5")
target <- '750c783e6ab0b503eaa86e310a5db738'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

current <- hmac(rep(0x0b, 16), 'Hi There', "md5")
target <- '9294727a3638bb1c13f48ef8158bfc9d'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

current <- hmac(rep(0xaa, 16), rep(0xdd, 50), "md5")
target <- '56be34521d144c88dbb8c733f0e8b3f6'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")


## test for key longer than block size. key should be hashed itself.
current <- hmac('ThisKeyIsLongerThan64BytesAndThereforeLongerThanTheHashDigestByFour', 'what do ya want for nothing?', "md5")
target <- '109af8ddd9cccbf8927d354da8be892b'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

## SHA1 tests inspired to the RFC 2104 and checked against the python
## hmac implementation.
current <- hmac('Jefe', 'what do ya want for nothing?', "sha1")
target <- 'effcdf6ae5eb2fa2d27416d5f184df9c259a7c79'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

current <- hmac(rep(0x0b, 16), 'Hi There', "sha1")
target <- '675b0b3a1b4ddf4e124872da6c2f632bfed957e9'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

current <- hmac(rep(0xaa, 16), rep(0xdd, 50), "sha1")
target <- 'd730594d167e35d5956fd8003d0db3d3f46dc7bb'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

## SHA256 tests from wikipedia
current <- hmac('key', 'The quick brown fox jumps over the lazy dog', 'sha256')
target <- 'f7bc83f430538424b13298e6aa6fb143ef4d59a14946175997479dbc2d1a3cd8'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

## SHA512 tests inspired, one from RFC 2104 and one from user, checked against python hmac
# python code:
# from hashlib import sha512
# import hmac
# print(hmac.new("love","message",sha512).hexdigest())

current <- hmac('Jefe', 'what do ya want for nothing?', "sha512")
target <- '164b7a7bfcf819e2e395fbe73b56e0a387bd64222e831fd610270cd7ea2505549758bf75c05a994a6d034f65f8f0e6fdcaeab1a34d4a6b4b636e070a38bce737'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

current <- hmac('love', 'message', 'sha512')
target <- 'f955821b3f6161673eb20985c677e3dc101860cafef3321ee31641acad9fcd85d9c7d3481ed3e4e1f4fa7af41d7e6ea9606d51e9bc7205d0091a4ee87d90fb9c'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")

## test for key longer than block size. key should be hashed itself.
current <- hmac('verylongkeyinfactlongerthanthedigestandthereforehashedbyitselfonlyifwegettothislengthsoonpleasepleasebutiddoesnotlooksoogoodahthereweare', 'message', 'sha512')
target <- '54b274484a8b8a371c8ae898f453d37c90d488b0c88c89dd705de06b18263d50c28069f1f778c8a997a61a4d53a06bc8e8719ad11a553c49140bb93a4636882e'
expect_true(identical(target, as.character(current)))
#cat(current, "\n")


## motivated by coverage report
rw <- charToRaw("123456789ABCDEF")
expect_true(all.equal(makeRaw(rw), rw))

current <- hmac('Jefe', 'what do ya want for nothing?', "md5", raw=TRUE)
target <- as.raw(c(0x75, 0x0c, 0x78, 0x3e, 0x6a, 0xb0, 0xb5, 0x03, 0xea, 0xa8, 0x6e, 0x31, 0x0a, 0x5d, 0xb7, 0x38))
expect_true(identical(target, current))

#digest:::padWithZeros(rw, "crc32")

##  AES -- Advanced Encryption Standard (AES) block cipher for R
##
##  Copyright (C) 2013 - 2019  Duncan Murdoch and Dirk Eddelbuettel
##  Copyright (C) 2019         Ion Suruceanu
##
##  This file is part of digest.
##
##  digest is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  digest is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with digest.  If not, see <http://www.gnu.org/licenses/>.

## This is loosely modelled on the AES code from the Python Crypto package.
## Currently only ECB, CBC, CFB and CTR modes are supported...

modes <- c("ECB", "CBC", "CFB", "PGP", "OFB", "CTR", "OPENPGP")

AES <- function(key, mode=c("ECB", "CBC", "CFB", "CTR"), IV=NULL, padding=FALSE) {
    mode <- match(match.arg(mode), modes)
    if (!(mode %in% c(1:3, 6)))
        stop("Only ECB, CBC, CFB and CTR mode encryption are supported.")	# #nocov
    if (padding && mode != 2)
        stop("Only CBC mode supports padding") # #nocov

    key <- as.raw(key)
    IV <- as.raw(IV)

    context <- .Call(AESinit, key)
    block_size <- 16
    key_size <- length(key)
    rm(key)

    encrypt <- function(text) {
        if (typeof(text) == "character")
            text <- charToRaw(text)
        if (mode == 1)
            .Call(AESencryptECB, context, text)
        else if (mode == 2) {
            if (padding) {
              bytes <- block_size - (length(text) %% block_size)
              text <- c(text, rep(as.raw(bytes), times = bytes))
            }

            len <- length(text)
            if (len %% block_size != 0)
                stop("Text length must be a multiple of ", block_size, " bytes, or use `padding=TRUE`")
            result <- raw(length(text))
            for (i in seq_len(len/block_size)) {
                ind <- (i-1)*block_size + seq(block_size)
                IV <<- .Call(AESencryptECB, context, xor(text[ind], IV))
                result[ind] <- IV
            }
            result
        } else if (mode == 3) {
            if (length(IV) != block_size) {
                stop("IV length must equal block size")
            }
            result <- raw(length(text))
            blocks <- split(text, ceiling(seq_along(text)/block_size))
            i <- 0
            for (b in blocks) {
                if (block_size == length(IV)) {
                    out <- .Call(AESencryptECB, context, IV)
                }
                len <- length(b)
                ind <- i*length(IV)+1:len
                i <- i + 1
                result[ind] <- xor(b, out[0:len])
                IV <<- result[ind]
            }
            result
        } else if (mode == 6) {
            len <- length(text)
            blocks <- (len + 15) %/% 16
            result <- raw(16*blocks)
            zero <- as.raw(0)
            for (i in 1:blocks) {
                result[16*(i-1)+1:16] <- IV
                byte <- 16
                repeat {
                    IV[byte] <<- as.raw((as.integer(IV[byte]) + 1) %% 256)
                    if (IV[byte] != zero || byte == 1) break
                    byte <- byte - 1
                }
            }
            result <- .Call(AESencryptECB, context, result)
            length(result) <- len
            xor(text, result)
        }
    }

    decrypt <- function(ciphertext, raw = FALSE) {
        if (mode == 1)
            result <- .Call(AESdecryptECB, context, ciphertext)
        else if (mode == 2) {
            len <- length(ciphertext)
            if (len %% 16 != 0)
                stop("Ciphertext length must be a multiple of 16 bytes")	# #nocov
            result <- raw(length(ciphertext))
            for (i in seq_len(len/16)) {
                ind <- (i-1)*16 + 1:16
                res <- .Call(AESdecryptECB, context, ciphertext[ind])
                result[ind] <- xor(res, IV)
                IV <<- ciphertext[ind]
            }
            if (padding) {
              bytes_to_remove <- as.integer(utils::tail(result, 1))
              result <- utils::head(result, -bytes_to_remove)
            }
        } else if (mode == 3) {
            if (length(IV) != block_size) {
                stop("IV length must equal block size")
            }
            result <- raw(length(ciphertext))
            blocks <- split(ciphertext, ceiling(seq_along(ciphertext)/block_size))
            i <- 0
            for (b in blocks) {
                if (block_size == length(IV)) {
                    out <- .Call(AESencryptECB, context, IV)
                }
                len <- length(b)
                ind <- i*length(IV)+1:len
                i <- i + 1
                result[ind] <- xor(b, out[0:len])
                IV <<- b
            }

        } else if (mode == 6)
              result <- encrypt(ciphertext)

        if (!raw)
            result <- rawToChar(result)						# #nocov
        result
    }

    structure(list(encrypt=encrypt,
                   decrypt=decrypt,
                   block_size=function() block_size,
                   IV=function() IV,
                   key_size=function() key_size,
                   mode=function() modes[mode]),
              class = "AES")
}

print.AES <- function(x, ...)							# #nocov
    cat("AES cipher object; mode", x$mode(), "key size", x$key_size(), "\n")	# #nocov

## tests for AES, taken from the examples in the manual page

suppressMessages(library(digest))

# FIPS-197 examples

hextextToRaw <- function(text) {
  vals <- matrix(as.integer(as.hexmode(strsplit(text, "")[[1]])), ncol=2, byrow=TRUE)
  vals <- vals %*% c(16, 1)
  as.raw(vals)
}

plaintext       <- hextextToRaw("00112233445566778899aabbccddeeff")

aes128key       <- hextextToRaw("000102030405060708090a0b0c0d0e0f")
aes128output    <- hextextToRaw("69c4e0d86a7b0430d8cdb78070b4c55a")
aes <- AES(aes128key)
#aes
aes128 <- aes$encrypt(plaintext)
#aes128
expect_true(identical(aes128, aes128output))
expect_true(identical(plaintext, aes$decrypt(aes128, raw=TRUE)))

aes192key       <- hextextToRaw("000102030405060708090a0b0c0d0e0f1011121314151617")
aes192output    <- hextextToRaw("dda97ca4864cdfe06eaf70a0ec0d7191")
aes <- AES(aes192key)
#aes
aes192 <- aes$encrypt(plaintext)
#aes192
expect_true(identical(aes192, aes192output))
expect_true(identical(plaintext, aes$decrypt(aes192, raw=TRUE)))

aes256key       <- hextextToRaw("000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f")
aes256output     <- hextextToRaw("8ea2b7ca516745bfeafc49904b496089")
aes <- AES(aes256key)
aes256 <- aes$encrypt(plaintext)
#aes256
expect_true(identical(aes256, aes256output))
expect_true(identical(plaintext, aes$decrypt(aes256, raw=TRUE)))

# SP800-38a examples

plaintext <- hextextToRaw(paste("6bc1bee22e409f96e93d7e117393172a",
                                "ae2d8a571e03ac9c9eb76fac45af8e51",
                                "30c81c46a35ce411e5fbc1191a0a52ef",
                                "f69f2445df4f9b17ad2b417be66c3710",sep=""))
key <- hextextToRaw("2b7e151628aed2a6abf7158809cf4f3c")

ecb128output <- hextextToRaw(paste("3ad77bb40d7a3660a89ecaf32466ef97",
                                   "f5d3d58503b9699de785895a96fdbaaf",
                                   "43b1cd7f598ece23881b00e3ed030688",
                                   "7b0c785e27e8ad3f8223207104725dd4",sep=""))
aes <- AES(key)
ecb128 <- aes$encrypt(plaintext)
#ecb128
expect_true(identical(ecb128, ecb128output))
expect_true(identical(plaintext, aes$decrypt(ecb128, raw=TRUE)))

cbc128output <- hextextToRaw(paste("7649abac8119b246cee98e9b12e9197d",
                                    "5086cb9b507219ee95db113a917678b2",
                                    "73bed6b8e3c1743b7116e69e22229516",
                                    "3ff1caa1681fac09120eca307586e1a7",sep=""))
iv <- hextextToRaw("000102030405060708090a0b0c0d0e0f")
aes <- AES(key, mode="CBC", IV=iv)
cbc128 <- aes$encrypt(plaintext)
#cbc128
expect_true(identical(cbc128, cbc128output))
aes <- AES(key, mode="CBC", IV=iv)
expect_true(identical(plaintext, aes$decrypt(cbc128, raw=TRUE)))

ctr128output <- hextextToRaw(paste("874d6191b620e3261bef6864990db6ce",
                                   "9806f66b7970fdff8617187bb9fffdff",
                                   "5ae4df3edbd5d35e5b4f09020db03eab",
                                   "1e031dda2fbe03d1792170a0f3009cee",sep=""))
iv <- hextextToRaw("f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff")
aes <- AES(key, mode="CTR", IV=iv)
ctr128 <- aes$encrypt(plaintext)
#ctr128
expect_true(identical(ctr128, ctr128output))
aes <- AES(key, mode="CTR", IV=iv)
expect_true(identical(plaintext, aes$decrypt(ctr128, raw=TRUE)))

#cfb
key <- hextextToRaw("2b7e151628aed2a6abf7158809cf4f3c")
iv <- hextextToRaw("000102030405060708090a0b0c0d0e0f")
#cfb not a multiplier of 16
text <- "This is very secret string"
key <- hextextToRaw("2b7e151628aed2a6abf7158809cf4f3c")
iv <- hextextToRaw("000102030405060708090a0b0c0d0e0f")

cfb128output <- hextextToRaw(paste("04960ebfb9044196ac6c4590bbdc8903",
                                   "e1259a479e199af94518",sep=""))
aes <- AES(key, mode="CFB", IV=iv)
cfb128 <- aes$encrypt(text)
expect_true(identical(cfb128, cfb128output))
aes <- AES(key, mode="CFB", IV=iv)
expect_true(identical(text, aes$decrypt(cfb128, raw=FALSE)))

#cfb128
cfb128output <- hextextToRaw(paste("3b3fd92eb72dad20333449f8e83cfb4a",
                                   "c8a64537a0b3a93fcde3cdad9f1ce58b",
                                   "26751f67a3cbb140b1808cf187a4f4df",
                                   "c04b05357c5d1c0eeac4c66f9ff7f2e6",sep=""))
aes <- AES(key, mode="CFB", IV=iv)
cfb128 <- aes$encrypt(plaintext)
expect_true(identical(cfb128, cfb128output))
aes <- AES(key, mode="CFB", IV=iv)
expect_true(identical(plaintext, aes$decrypt(cfb128, raw=TRUE)))

# test throws exception on IV null or not a multiplier of 16 bytes
aes <- AES(key, mode="CFB", IV=NULL)
expect_error(aes$encrypt(plaintext))
expect_error(aes$decrypt(plaintext))

aes <- AES(key, mode="CFB", IV=raw(15))
expect_error(aes$encrypt(plaintext))
expect_error(aes$decrypt(plaintext))

# test that providing raw vs character inputs results in the same output
text <- "0123456789ABCDEF"
raw_text <- charToRaw(text)
expect_identical(
  AES(key, mode="ECB")$encrypt(text),
  AES(key, mode="ECB")$encrypt(raw_text)
)

# test padding for CBC
expect_length(AES(raw_text, "CBC", raw_text)$encrypt(text), 16)
expect_length(AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt(text), 32)
expect_identical(
  AES(raw_text, "CBC", raw_text)$encrypt(text),
  head(AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt(text), 16)
)
expect_identical(
  paste(AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt(text), collapse = ""),
  "9d2cda901b682d3359709a5ab24196242125333bdf540132179ac0de79e1837a"
)

cipher <- AES(raw_text, "CBC", raw_text)$encrypt(text)
expect_identical(AES(raw_text, "CBC", raw_text)$decrypt(cipher), text)
cipher_padded <- AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt(text)
expect_identical(AES(raw_text, "CBC", raw_text, padding = TRUE)$decrypt(cipher_padded), text)

expect_error(AES(raw_text, "CBC", raw_text)$encrypt("testing"))
expect_identical(
  paste(AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt("testing"), collapse = ""),
  "4017d340b6eafbb66c3dfceb10808cff"
)
expect_identical(
  AES(raw_text, "CBC", raw_text, padding = TRUE)$decrypt(
    AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt("testing")
  ),
  "testing"
)

expect_error(AES(raw_text, "CBC", raw_text)$encrypt("testingalongerstring"))
expect_identical(
  paste(AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt("testingalongerstring"), collapse = ""),
  "2d870ada654f4bdff182497a88e08013a232bfb5701c12a4d3b29f4f66e74ecd"
)
expect_identical(
  AES(raw_text, "CBC", raw_text, padding = TRUE)$decrypt(
    AES(raw_text, "CBC", raw_text, padding = TRUE)$encrypt("testingalongerstring")
  ),
  "testingalongerstring"
)

##  digest -- hash digest functions for R
##
##  Copyright (C) 2003 - 2023  Dirk Eddelbuettel <edd@debian.org>
##
##  This file is part of digest.
##
##  digest is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  digest is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with digest.  If not, see <http://www.gnu.org/licenses/>.

.pkgenv <- new.env(parent=emptyenv())

.onLoad <- function(libname, pkgname) {
    ## we set a default level of two, with a possible override
    .pkgenv[["serializeVersion"]] <- getOption("serializeVersion", 2L)  # #nocov start
    ## allow old crc32 behaviour
    .pkgenv[["crc32Preference"]] <- getOption("digestOldCRC32Format", FALSE)
    ## allow version specific sha1 behaviour
    .pkgenv[["sha1PackageVersion"]] <- getOption("sha1PackageVersion",
                                                 packageVersion("digest"))
    ## cache if we are on Windows as the call is a little expensive (GH issue #137)
    .pkgenv[["isWindows"]] <- Sys.info()[["sysname"]] == "Windows"

    ## cache if serialize() supports 'nosharing'
    .pkgenv[["hasNoSharing"]] <- "nosharing" %in% names(formals(serialize)) # #nocov end
}

.getSerializeVersion <- function() {
    ## return the options() value if set, otherwise the package env value
    ## doing it as a two-step ensure we can set a different default later
    getOption("serializeVersion", .pkgenv[["serializeVersion"]])
}

.getCRC32PreferOldOutput <- function() {
    ## return the options() value if set, otherwise the package env value
    ## doing it as a two-step ensure we can set a different default later
    getOption("digestOldCRC32Format", .pkgenv[["crc32Preference"]])
}

.getsha1PackageVersion <- function() {
    ## return the options() value if set, otherwise the package env value
    ## doing it as a two-step ensure we can set a different default later
    package_version(
        getOption("sha1PackageVersion", .pkgenv[["sha1PackageVersion"]])
    )
}

.isWindows <- function() {
    ## return the cached value of Sys.info()[["sysname"]] == "Windows"
    .pkgenv[["isWindows"]]
}

.hasNoSharing <- function() {
    ## return the cached value of "nosharing" %in% names(formals(serialize))
    .pkgenv[["hasNoSharing"]]
}

## tests for old versus new (ie guaranteed eight chars, added in 0.6.16) format for crc32

suppressMessages(library(digest))


args <- c(0L, 51L, 126L, 8480L, 60929L, 180832L)

resOld <- c("b7fa0888",  "82a699e",   "4754b3",    "b3da3",     "e67c",      "872")
resNew <- c("b7fa0888", "082a699e", "004754b3", "000b3da3", "0000e67c", "00000872")

options("digestOldCRC32Format" = TRUE)
expect_identical(sapply(args, digest, algo="crc32"), resOld)

options("digestOldCRC32Format" = FALSE)
expect_identical(sapply(args, digest, algo="crc32"), resNew)

if (requireNamespace("tinytest", quietly=TRUE)) {

    ## Set a seed to make the test deterministic
    set.seed(42)

    ## there are several more granular ways to test files in a
    ## tinytest directory, see its package vignette; tests can also
    ## run once the package is installed using the same command

    ## we need version 0.9.3 or later
    if (packageVersion("tinytest") >= "0.9.3") {
        ## expect_length is in tinytest 1.4.1
        if (!"expect_length" %in% getNamespaceExports("tinytest")) {
            expect_length <- function(x, n) tinytest::expect_equal(length(x), n)
        }
        tinytest::test_package("digest")
    }
}

suppressMessages(library(digest))

input <- letters[1:5]
hashed_seed_0 <- c(-902917054L, 14385563L, -289776295L, 592496261L, 286663184L)
expect_equal(digest2int(input), hashed_seed_0)

hashed_seed_1 <- c(14385563L, -289776295L, 592496261L, 286663184L, 1208324078L)
expect_equal(digest2int(input, 1L), hashed_seed_1)

# should fail if uint32_t on the system is not a 32-bit unsigned integer
expect_equal(digest2int("cat sat on the mat"), 562079877L)
expect_equal(digest2int("The quick brown fox jumps over the lazy dog"), 1369346549L)

<!--
%\VignetteIndexEntry{sha1() versus digest()}
%\VignetteEngine{simplermarkdown::mdweave_to_html}
%\VignetteEncoding{UTF-8}
-->
---
title: "Calculating SHA1 hashes with digest() and sha1()"
author: "Thierry Onkelinx and Dirk Eddelbuettel"
date: "Written Jan 2016, updated Jan 2018 and Oct 2020"
css: "water.css"
---

NB: This vignette is (still) work-in-progress and not yet complete.

## Short intro on hashes

TBD

## Difference between `digest()` and `sha1()`

R [FAQ 7.31](https://cran.r-project.org/doc/FAQ/R-FAQ.html#Why-doesn_0027t-R-think-these-numbers-are-equal_003f) illustrates potential problems with floating point arithmetic. Mathematically the equality $x = \sqrt{x}^2$ should hold. But the precision of floating points numbers is finite. Hence some rounding is done, leading to numbers which are no longer identical.

An illustration:

```{#faq7_31 .R} 
# FAQ 7.31
a0 <- 2
b <- sqrt(a0)
a1 <- b ^ 2
identical(a0, a1)
a0 - a1
a <- c(a0, a1)
# hexadecimal representation
sprintf("%a", a)
```

Although the difference is small, any difference will result in different hash when using the `digest()` function. 
However, the `sha1()` function tackles this problem by using the hexadecimal representation of the numbers and truncates 
that representation to a certain number of digits prior to calculating the hash function. 

```{#faq7_31digest .R}
library(digest)
# different hashes with digest
sapply(a, digest, algo = "sha1")
# same hash with sha1 with default digits (14)
sapply(a, sha1)
# larger digits can lead to different hashes
sapply(a, sha1, digits = 15)
# decreasing the number of digits gives a stronger truncation
# the hash will change when then truncation gives a different result
# case where truncating gives same hexadecimal value
sapply(a, sha1, digits = 13)
sapply(a, sha1, digits = 10)
# case where truncating gives different hexadecimal value
c(sha1(pi), sha1(pi, digits = 13), sha1(pi, digits = 10))
```

The result of floating point arithematic on 32-bit and 64-bit can be slightly different. E.g. `print(pi ^ 11, 22)` returns `294204.01797389047` on 32-bit and `294204.01797389053` on 64-bit. Note that only the last 2 digits are different.  

| command | 32-bit | 64-bit|
| - | - | - |
| `print(pi ^ 11, 22)` | `294204.01797389047` | `294204.01797389053` |
| `sprintf("%a", pi ^ 11)`| `"0x1.1f4f01267bf5fp+18"` | `"0x1.1f4f01267bf6p+18"` |
| `digest(pi ^ 11, algo = "sha1")` | `"c5efc7f167df1bb402b27cf9b405d7cebfba339a"` | `"b61f6fea5e2a7952692cefe8bba86a00af3de713"`|
| `sha1(pi ^ 11, digits = 14)` | `"5c7740500b8f78ec2354ea6af58ea69634d9b7b1"` | `"4f3e296b9922a7ddece2183b1478d0685609a359"` |
| `sha1(pi ^ 11, digits = 13)` | `"372289f87396b0877ccb4790cf40bcb5e658cad7"` | `"372289f87396b0877ccb4790cf40bcb5e658cad7"` |
| `sha1(pi ^ 11, digits = 10)` | `"c05965af43f9566bfb5622f335817f674abfc9e4"` | `"c05965af43f9566bfb5622f335817f674abfc9e4"` |

## Choosing `digest()` or `sha1()`

TBD

## Creating a sha1 method for other classes

### How to

1. Identify the relevant components for the hash.
1. Determine the class of each relevant component and check if they are handled by `sha1()`.
    - Write a method for each component class not yet handled by `sha1`.
1. Extract the relevant components.
1. Combine the relevant components into a list. Not required in case of a single component.
1. Apply `sha1()` on the (list of) relevant component(s).
1. Turn this into a function with name sha1._classname_.
1. sha1._classname_ needs exactly the same arguments as `sha1()`
1. Choose sensible defaults for the arguments
    - `zapsmall = 7` is recommended.
    - `digits = 14` is recommended in case all numerics are data.
    - `digits = 4` is recommended in case some numerics stem from floating point arithmetic.

###  summary.lm

Let's illustrate this using the summary of a simple linear regression. Suppose that we want a hash that takes into account the coefficients, their standard error and sigma.

```{#sha1_lm_sum .R}
# taken from the help file of lm.influence
lm_SR <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LifeCycleSavings)
lm_sum <- summary(lm_SR)
class(lm_sum)
# str() gives the structure of the lm object
str(lm_sum)
# extract the coefficients and their standard error
coef_sum <- coef(lm_sum)[, c("Estimate", "Std. Error")]
# extract sigma
sigma <- lm_sum$sigma
# check the class of each component
class(coef_sum)
class(sigma)
# sha1() has methods for both matrix and numeric
# because the values originate from floating point arithmetic it is better to use a low number of digits
sha1(coef_sum, digits = 4)
sha1(sigma, digits = 4)
# we want a single hash
# combining the components in a list is a solution that works
sha1(list(coef_sum, sigma), digits = 4)
# now turn everything into an S3 method
#   - a function with name "sha1.classname"
#   - must have the same arguments as sha1()
sha1.summary.lm <- function(x, digits = 4, zapsmall = 7){
    coef_sum <- coef(x)[, c("Estimate", "Std. Error")]
    sigma <- x$sigma
    combined <- list(coef_sum, sigma)
    sha1(combined, digits = digits, zapsmall = zapsmall)
}
sha1(lm_sum)

# try an altered dataset
LCS2 <- LifeCycleSavings[rownames(LifeCycleSavings) != "Zambia", ]
lm_SR2 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LCS2)
sha1(summary(lm_SR2))
```

###  lm

Let's illustrate this using the summary of a simple linear regression. Suppose that we want a hash that takes into account the coefficients, their standard error and sigma.

```{#sha1_lm .R}
class(lm_SR)
# str() gives the structure of the lm object
str(lm_SR)
# extract the model and the terms
lm_model <- lm_SR$model
lm_terms <- lm_SR$terms
# check their class
class(lm_model) # handled by sha1()
class(lm_terms) # not handled by sha1()
# define a method for formula
sha1.formula <- function(x, digits = 14, zapsmall = 7, ..., algo = "sha1"){
    sha1(as.character(x), digits = digits, zapsmall = zapsmall, algo = algo)
}
sha1(lm_terms)
sha1(lm_model)
# define a method for lm
sha1.lm <- function(x, digits = 14, zapsmall = 7, ..., algo = "sha1"){
    lm_model <- x$model
    lm_terms <- x$terms
    combined <- list(lm_model, lm_terms)
    sha1(combined, digits = digits, zapsmall = zapsmall, ..., algo = algo)
}
sha1(lm_SR)
sha1(lm_SR2)
```

## Using hashes to track changes in analysis

Use case

- automated analysis
- update frequency of the data might be lower than the frequency of automated analysis
- similar analyses on many datasets (e.g. many species in ecology)
- analyses that require a lot of computing time
    - not rerunning an analysis because nothing has changed saves enough resources to compensate the overhead of tracking changes

- Bundle all relevant information on an analysis in a class
    - data
    - method
    - formula
    - other metadata
    - resulting model
- calculate `sha1()`

    file fingerprint
      ~ `sha1()` on the stable parts
    
    status fingerprint
      ~ `sha1()` on the parts that result for the model

1. Prepare analysis objects
1. Store each analysis object in a rda file which uses the file fingerprint as filename
    - File will already exist when no change in analysis
    - Don't overwrite existing files
1. Loop over all rda files
    - Do nothing if the analysis was run
    - Otherwise run the analysis and update the status and status fingerprint

\name{digest2int}
\alias{digest2int}
\title{hash arbitrary string to integer}
\description{
  The \code{digest2int} function calculates integer hash of an arbitrary string.
  This is useful for randomized experiments, feature hashing, etc.
}
\usage{
digest2int(x, seed = 0L)
}
\arguments{
  \item{x}{An arbitrary character vector.}
  \item{seed}{an integer for algorithm initial state.
  Function will produce different hashes for same input and different seed values.}
}
\value{
  The \code{digest2int} function returns integer vector of the same length
  as input vector \code{x}.
}
\references{
  Jenkins's \code{one_at_a_time} hash:
  \url{https://en.wikipedia.org/wiki/Jenkins_hash_function#one_at_a_time}.
}
\author{Dmitriy Selivanov \email{selivanov.dmitriy@gmail.com} for the \R interface;
    Bob Jenkins for original implementation
    \url{http://www.burtleburtle.net/bob/hash/doobs.html}
}
\seealso{\code{\link{digest}}}

\examples{

current <- digest2int("The quick brown fox jumps over the lazy dog", 0L)
target <- 1369346549L
stopifnot(identical(target, current))

}
\keyword{misc}

\docType{methods}
\name{sha1}
\alias{sha1}
\alias{sha1_attr_digest}
\alias{sha1_digest}
\alias{sha1.array}
\alias{sha1.integer}
\alias{sha1.numeric}
\alias{sha1.character}
\alias{sha1.complex}
\alias{sha1.Date}
\alias{sha1.factor}
\alias{sha1.NULL}
\alias{sha1.logical}
\alias{sha1.matrix}
\alias{sha1.data.frame}
\alias{sha1.list}
\alias{sha1.pairlist}
\alias{sha1.name}
\alias{sha1.POSIXlt}
\alias{sha1.POSIXct}
\alias{sha1.anova}
\alias{sha1.function}
\alias{sha1.call}
\alias{sha1.raw}
\alias{sha1.formula}
\alias{sha1.(}
\alias{sha1.environment}
\alias{sha1.<-}
\title{Calculate a SHA1 hash of an object}
\author{Thierry Onkelinx}
\usage{
sha1(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{numeric}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{complex}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{Date}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{matrix}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{data.frame}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{array}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{list}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{pairlist}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{POSIXlt}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{POSIXct}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{anova}(x, digits = 4, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{function}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")

\method{sha1}{formula}(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{`(`}(...)

sha1_digest(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{NULL}(...)
\method{sha1}{name}(...)

sha1_attr_digest(x, digits = 14, zapsmall = 7, ..., algo = "sha1")
\method{sha1}{call}(...)
\method{sha1}{character}(...)
\method{sha1}{factor}(...)
\method{sha1}{integer}(...)
\method{sha1}{logical}(...)
\method{sha1}{raw}(...)
\method{sha1}{environment}(...)
\method{sha1}{`<-`}(...)
}
\arguments{
\item{x}{the object to calculate the SHA1}

\item{digits}{the approximate number of significant digits in base 10. Will
be converted to a base 16 equivalent. Defaults to \code{digits = 14}, except for
sha1.anova where \code{digits = 4}}

\item{zapsmall}{the approximate negative magnitude of the smallest relevant
digit. Will be converted to a base 2 equivalent. Values smaller than this
number are equivalent to 0. Defaults to \code{zapsmall = 7}}

\item{...}{If it is the only defined argument, passed to another \code{sha1}
method.  If other arguments exist, see Details for usage.}

\item{algo}{The hashing algorithm to be used by \code{\link{digest}}. Defaults to
"sha1"}
}
\description{
Calculate a SHA1 hash of an object. The main difference with
\code{digest(x, algo = "sha1")} is that \code{sha1()} will give the same hash on
32-bit and 64-bit systems. Note that the results depends on the setting of
\code{digits} and \code{zapsmall} when handling floating point numbers. The
current defaults keep \code{digits} and \code{zapsmall} as large as possible
while maintaining the same hash on 32 bit and 64 bit systems.
}
\details{
\code{sha1_digest()} is a convenience function for objects where attributes
cannot be added to apply the \code{digest()} function to its arguments.
\code{sha1_attr_digest()} is a convenience function for objects where objects
can be added to generate the hash.  If generating hashes for objects in other
packages, one of these two functions is recommended for use (typically,
\code{sha1_attr_digest()}).

Extra arguments:

environment: An optional extra argument for \code{sha1.function} and
\code{sha1.formula} should be TRUE, FALSE or missing. \code{sha1.function} and
\code{sha1.formula} will ignore the environment of the function only when
\code{environment = FALSE}.
}
\note{
\code{sha1} gained an \code{algo} argument since version 0.6.15. This allows
\code{sha1()} to use all hashing algorithms available in \code{digest()}. The
hashes created with \code{sha1(x)} from digest >= 0.6.15 are identical to
\code{sha1(x)} from digest <= 0.6.14. The only exceptions are hashes created
with \code{sha1(x, algo = "sha1")}, they will be different starting from digest
0.6.15

Until version 0.6.22, \code{sha1} ignored the attributes of the object for
some classes. This was fixed in version 0.6.23. Use
\code{options(sha1PackageVersion = "0.6.22")} to get the old behaviour.

Version 0.6.24 and later ignore attributes named \code{srcref}.

Version 0.6.38 and later use \code{digest()} as a fallback for undefined
methods. Note that this breaks the guarantee for identical hashes between 32-bit
and 64-bit system. We recommend to write a custom method for such class. The
basic idea is the convert the class in a list of classes that \code{sha1} can
handle. The project \url{https://github.com/inbo/n2kanalysis/} demonstrates
how to create custom \code{sha1} dispatchers for other S3 classes, see
file \url{https://github.com/inbo/n2kanalysis/blob/main/R/sha1.R}.

}

\name{AES}
\alias{AES}

\title{
Create AES block cipher object
}
\description{
This creates an object that can perform the Advanced Encryption
Standard (AES) block cipher.
}
\usage{
AES(key, mode=c("ECB", "CBC", "CFB", "CTR"), IV=NULL, padding=FALSE)
}
\arguments{
  \item{key}{
The key as a 16, 24 or 32 byte raw vector for AES-128, AES-192 or
AES-256 respectively.
}
  \item{mode}{
The encryption mode to use.  Currently only \dQuote{electronic
codebook} (ECB), \dQuote{cipher-block chaining} (CBC), \dQuote{cipher feedback} (CFB) and
\dQuote{counter} (CTR) modes are supported.
}
  \item{IV}{
The initial vector for CBC and CFB mode or initial counter for CTR mode.
}
  \item{padding}{
Whether or not PKCS#7 padding is used during encryption and decryption in CBC mode.
}
}
\value{
An object of class \code{"AES"}.  This is a list containing the
following component functions:

\item{encrypt(text)}{A function to encrypt a text vector.  The text
may be a single element character vector or a raw vector.  It returns
the ciphertext as a raw vector.}

\item{decrypt(ciphertext, raw = FALSE)}{A function to decrypt the
ciphertext. In ECB mode, the same AES
object can be used for both encryption and decryption, but in
CBC, CFB and CTR modes a new object needs to be
created, using the same initial \code{key} and \code{IV} values.
Returns a single element character vector, or a raw vector if \code{raw = TRUE}.}

\item{IV()}{Report on the current state of the initialization vector.
As blocks are encrypted or decrypted in CBC, CFB or CTR mode, the initialization
vector is updated, so both operations can be performed sequentially on
subsets of the text or ciphertext.}

\item{block_size(), key_size(), mode()}{Report on these aspects of
the AES object.}

}
\details{
The standard NIST definition of CTR mode doesn't define how the counter
is updated, it just requires that it be updated with each block
and not repeat itself for a long time.  This implementation treats it as a
128 bit integer and adds 1 with each successive block.
}
\author{
The R interface was written by Duncan Murdoch. The design is loosely
based on the Python Crypto implementation.  The underlying AES
implementation is by Christophe Devine.
}
\references{
United States National Institute of Standards and Technology (2001).
"Announcing the ADVANCED ENCRYPTION STANDARD (AES)".
Federal Information Processing Standards Publication 197.
\url{https://csrc.nist.gov/publications/fips/fips197/fips-197.pdf}.

Morris Dworkin (2001).
"Recommendation for Block Cipher Modes of Operation".
NIST Special Publication 800-38A 2001 Edition.
}
\examples{
# First in ECB mode: the repeated block is coded the same way each time
msg <- as.raw(c(1:16, 1:16))
key <- as.raw(1:16)
aes <- AES(key, mode="ECB")
aes$encrypt(msg)
aes$decrypt(aes$encrypt(msg), raw=TRUE)

# Now in CBC mode:  each encoding is different
iv <- sample(0:255, 16, replace=TRUE)
aes <- AES(key, mode="CBC", iv)
code <- aes$encrypt(msg)
code

# Need a new object for decryption in CBC mode
aes <- AES(key, mode="CBC", iv)
aes$decrypt(code, raw=TRUE)

# In CBC mode, the input length must be a multiple of 16 bytes.
# You can use `padding = TRUE` to ensure the input length is always valid.
AES(key, mode="CBC", iv, padding = TRUE)$encrypt(as.raw(1:15))

# CFB mode: IV must be the same length as the Block's block size
# Two different instances of AES are required for encryption and decryption
iv <- sample(0:255, 16, replace=TRUE)
aes <- AES(key, mode="CFB", iv)
code <- aes$encrypt(msg)
code
#decrypt
aes <-  AES(key, mode="CFB", iv)
aes$decrypt(code)


# FIPS-197 examples

hextextToRaw <- function(text) {
  vals <- matrix(as.integer(as.hexmode(strsplit(text, "")[[1]])), ncol=2, byrow=TRUE)
  vals <- vals \%*\% c(16, 1)
  as.raw(vals)
}

plaintext       <- hextextToRaw("00112233445566778899aabbccddeeff")

aes128key       <- hextextToRaw("000102030405060708090a0b0c0d0e0f")
aes128output    <- hextextToRaw("69c4e0d86a7b0430d8cdb78070b4c55a")
aes <- AES(aes128key)
aes128 <- aes$encrypt(plaintext)
stopifnot(identical(aes128, aes128output))
stopifnot(identical(plaintext, aes$decrypt(aes128, raw=TRUE)))

aes192key       <- hextextToRaw("000102030405060708090a0b0c0d0e0f1011121314151617")
aes192output    <- hextextToRaw("dda97ca4864cdfe06eaf70a0ec0d7191")
aes <- AES(aes192key)
aes192 <- aes$encrypt(plaintext)
stopifnot(identical(aes192, aes192output))
stopifnot(identical(plaintext, aes$decrypt(aes192, raw=TRUE)))

aes256key       <- hextextToRaw("000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f")
aes256output     <- hextextToRaw("8ea2b7ca516745bfeafc49904b496089")
aes <- AES(aes256key)
aes256 <- aes$encrypt(plaintext)
stopifnot(identical(aes256, aes256output))
stopifnot(identical(plaintext, aes$decrypt(aes256, raw=TRUE)))

# SP800-38a examples

plaintext <- hextextToRaw(paste("6bc1bee22e409f96e93d7e117393172a",
                                "ae2d8a571e03ac9c9eb76fac45af8e51",
                                "30c81c46a35ce411e5fbc1191a0a52ef",
                                "f69f2445df4f9b17ad2b417be66c3710",sep=""))
key <- hextextToRaw("2b7e151628aed2a6abf7158809cf4f3c")

ecb128output <- hextextToRaw(paste("3ad77bb40d7a3660a89ecaf32466ef97",
                                   "f5d3d58503b9699de785895a96fdbaaf",
                                   "43b1cd7f598ece23881b00e3ed030688",
                                   "7b0c785e27e8ad3f8223207104725dd4",sep=""))
aes <- AES(key)
ecb128 <- aes$encrypt(plaintext)
stopifnot(identical(ecb128, ecb128output))
stopifnot(identical(plaintext, aes$decrypt(ecb128, raw=TRUE)))

cbc128output <- hextextToRaw(paste("7649abac8119b246cee98e9b12e9197d",
                                   "5086cb9b507219ee95db113a917678b2",
                                   "73bed6b8e3c1743b7116e69e22229516",
                                   "3ff1caa1681fac09120eca307586e1a7",sep=""))
iv <- hextextToRaw("000102030405060708090a0b0c0d0e0f")
aes <- AES(key, mode="CBC", IV=iv)
cbc128 <- aes$encrypt(plaintext)
stopifnot(identical(cbc128, cbc128output))
aes <- AES(key, mode="CBC", IV=iv)
stopifnot(identical(plaintext, aes$decrypt(cbc128, raw=TRUE)))


cfb128output <- hextextToRaw(paste("3b3fd92eb72dad20333449f8e83cfb4a",
                                   "c8a64537a0b3a93fcde3cdad9f1ce58b",
                                   "26751f67a3cbb140b1808cf187a4f4df",
                                   "c04b05357c5d1c0eeac4c66f9ff7f2e6",sep=""))
aes <- AES(key, mode="CFB", IV=iv)
cfb128 <- aes$encrypt(plaintext)
stopifnot(identical(cfb128, cfb128output))
aes <- AES(key, mode="CFB", IV=iv)
stopifnot(identical(plaintext, aes$decrypt(cfb128, raw=TRUE)))


ctr128output <- hextextToRaw(paste("874d6191b620e3261bef6864990db6ce",
                                   "9806f66b7970fdff8617187bb9fffdff",
                                   "5ae4df3edbd5d35e5b4f09020db03eab",
                                   "1e031dda2fbe03d1792170a0f3009cee",sep=""))
iv <- hextextToRaw("f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff")
aes <- AES(key, mode="CTR", IV=iv)
ctr128 <- aes$encrypt(plaintext)
stopifnot(identical(ctr128, ctr128output))
aes <- AES(key, mode="CTR", IV=iv)
stopifnot(identical(plaintext, aes$decrypt(ctr128, raw=TRUE)))

}

\name{getVDigest}
\alias{getVDigest}
\title{Set a vectorised function for creating hash function digests}
\description{
The \code{getVDigest} function extends \code{digest} by allowing one to
set a function that returns hash summaries as a character vector of the same length as the input. It also provides a performance advantage
when repeated calls are necessary (e.g. applying a hash function repeatedly to an object). The returned function contains the same arguments as \code{digest} with the exception of the \code{raw} argument (see \code{\link{digest}} for more details).
}
\usage{
getVDigest(algo=c("md5", "sha1", "crc32", "sha256", "sha512", "xxhash32",
                  "xxhash64", "murmur32", "spookyhash", "blake3", "crc32c",
                  "xxh3_64", "xxh3_128"),
             errormode=c("stop","warn","silent"))
}
\arguments{
  \item{algo}{The algorithms to be used; currently available choices are
    \code{md5}, which is also the default, \code{sha1}, \code{crc32},
    \code{sha256}, \code{sha512}, \code{xxhash32}, \code{xxhash64},
    \code{murmur32}, \code{spookyhash}, \code{blake3}, \code{crc32c},
    \code{xxh3_64}, and \code{xxh3_128}.}
  \item{errormode}{A character value denoting a choice for the behaviour in
    the case of error: \sQuote{stop} aborts (and is the default value),
    \sQuote{warn} emits a warning and returns \code{NULL} and
    \sQuote{silent} suppresses the error and returns an empty string.}
}
\value{
  The \code{getVDigest} function returns a function for a given
  algorithm and error-mode.
}
\details{
 Note that since one hash summary will be returned for each element passed as input,  care must be taken when determining whether or not to include the data structure as  part of the object. For instance, to return the equivalent output of
 \code{digest(list("a"))} it would be necessary to wrap the list object itself
 \code{getVDigest()(list(list("a")))}
}
\seealso{\code{\link{digest}}, \code{\link{serialize}}, \code{\link{md5sum}}}
\examples{
stretch_key <- function(d, n) {
    md5 <- getVDigest()
    for (i in seq_len(n))
        d <- md5(d, serialize = FALSE)
    d
}
stretch_key('abc123', 65e3)
sha1 <- getVDigest(algo = 'sha1')
sha1(letters)

md5Input <-
    c("",
      "a",
      "abc",
      "message digest",
      "abcdefghijklmnopqrstuvwxyz",
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
      paste("12345678901234567890123456789012345678901234567890123456789012",
            "345678901234567890", sep=""))
md5Output <-
    c("d41d8cd98f00b204e9800998ecf8427e",
      "0cc175b9c0f1b6a831c399e269772661",
      "900150983cd24fb0d6963f7d28e17f72",
      "f96b697d7cb7938d525a2f31aaf161d0",
      "c3fcd3d76192e4007dfb496cca67e13b",
      "d174ab98d277d9f5a5611c2c9f419d9f",
      "57edf4a22be3c955ac49da2e2107b67a")

md5 <- getVDigest()
stopifnot(identical(md5(md5Input, serialize = FALSE), md5Output))
stopifnot(identical(digest(list("abc")),
                 md5(list(list("abc")))))

sha512Input <-c(
    "",
    "The quick brown fox jumps over the lazy dog."
    )
sha512Output <- c(
    paste0("cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce",
           "47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e"),
    paste0("91ea1245f20d46ae9a037a989f54f1f790f0a47607eeb8a14d12890cea77a1bb",
           "c6c7ed9cf205e67b7f2b8fd4c7dfd3a7a8617e45f3c463d481c7e586c39ac1ed")
    )

sha512 <- getVDigest(algo = 'sha512')
stopifnot(identical(sha512(sha512Input, serialize = FALSE), sha512Output))

}
\keyword{misc}

\name{hmac}
\alias{hmac}
\title{compute a hash-based message authentication code}
\description{
  The \code{hmac} function calculates a message authentication code
  (MAC) involving the specified cryptographic hash function in
  combination with a given secret key.
}
\usage{
hmac(key, object,
     algo = c("md5", "sha1", "crc32", "sha256", "sha512"),
     serialize = FALSE, raw = FALSE, ...)
}
\arguments{
  \item{key}{An arbitrary character or numeric vector, to use as
    pre-shared secret key.}
  \item{object}{An arbitrary R object which will then be passed to the
    \code{\link{serialize}} function, unless the \code{serialize}
    argument is set to \code{FALSE}.}
  \item{algo}{The algorithms to be used; currently available choices are
    \code{md5}, which is also the default, \code{sha1}, \code{crc32} and
    \code{sha256}.}
  \item{serialize}{default value of \code{serialize} is here FALSE, not
    TRUE as it is in \code{digest}.}
  \item{raw}{This flag alters the type of the output.  Setting this to
    \code{TRUE} causes the function to return an object of type
    \code{"raw"} instead of \code{"character"}.} 
  \item{...}{All remaining arguments are passed to \code{digest}.  }
}
\value{
  The \code{hmac} function uses the \code{digest} to return a hash
  digest as specified in the RFC 2104.
}
\references{
  MD5: \url{https://www.ietf.org/rfc/rfc1321.txt}. 

  SHA-1: \url{https://en.wikipedia.org/wiki/SHA-1}.
  SHA-256: \url{https://csrc.nist.gov/publications/fips/fips180-2/fips180-2withchangenotice.pdf}.
  CRC32:  The original reference webpage at \code{rocksoft.com} has
  vanished from the web; see
  \url{https://en.wikipedia.org/wiki/Cyclic_redundancy_check} for
  general information on CRC algorithms.

  \url{https://aarongifford.com/computers/sha.html} for the 
  integrated C implementation of sha-512.

  The page for the code underlying the C functions used here for sha-1
  and md5, and further references, is no longer accessible.  Please see
  \url{https://en.wikipedia.org/wiki/SHA-1} and
  \url{https://en.wikipedia.org/wiki/MD5}.  

  \url{https://zlib.net} for documentation on the zlib library which
  supplied the code for crc32.

  \url{https://en.wikipedia.org/wiki/SHA_hash_functions} for
  documentation on the sha functions.
}
\author{Mario Frasca \email{mfrasca@zonnet.nl}.} 
\seealso{\code{\link{digest}}}

\examples{



## Standard RFC 2104 test vectors
current <- hmac('Jefe', 'what do ya want for nothing?', "md5")
target <- '750c783e6ab0b503eaa86e310a5db738'
stopifnot(identical(target, as.character(current)))

current <- hmac(rep(0x0b, 16), 'Hi There', "md5")
target <- '9294727a3638bb1c13f48ef8158bfc9d'
stopifnot(identical(target, as.character(current)))

current <- hmac(rep(0xaa, 16), rep(0xdd, 50), "md5")
target <- '56be34521d144c88dbb8c733f0e8b3f6'
stopifnot(identical(target, as.character(current)))

## SHA1 tests inspired to the RFC 2104 and checked against the python
## hmac implementation.
current <- hmac('Jefe', 'what do ya want for nothing?', "sha1")
target <- 'effcdf6ae5eb2fa2d27416d5f184df9c259a7c79'
stopifnot(identical(target, as.character(current)))

current <- hmac(rep(0x0b, 16), 'Hi There', "sha1")
target <- '675b0b3a1b4ddf4e124872da6c2f632bfed957e9'
stopifnot(identical(target, as.character(current)))

current <- hmac(rep(0xaa, 16), rep(0xdd, 50), "sha1")
target <- 'd730594d167e35d5956fd8003d0db3d3f46dc7bb'
stopifnot(identical(target, as.character(current)))

}
\keyword{misc}

\name{makeRaw}
\alias{makeRaw}
\alias{makeRaw.default}
\alias{makeRaw.raw}
\alias{makeRaw.character}
\alias{makeRaw.digest}
\title{Create a raw object}
\usage{
makeRaw(object)

\method{makeRaw}{raw}(object)

\method{makeRaw}{character}(object)

\method{makeRaw}{digest}(object)

\method{makeRaw}{raw}(object)
}
\arguments{
  \item{object}{The object to convert into a \code{raw} vector}
}
\description{
  A helper function used to create \code{raw} methods.
}
\value{
  A \code{raw} vector is returned.
}
\author{Dirk Eddelbuettel}
\examples{
makeRaw("1234567890ABCDE")
}

\name{digest}
\alias{digest}
\title{Create hash function digests for arbitrary R objects or files}
\description{
  The \code{digest} function applies one of several cryptographic or
  non-cryptographics hash function to arbitrary R objects or files.  By
  default, the objects are internally serialized, and the selected hash
  functions algorithms can be used to compute a compact digest of the
  serialized object.

  In order to compare this implementation with others, serialization of
  the input argument can also be turned off in which the input argument
  must be a character string for which its digest is returned.
}
\usage{
digest(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
                      "xxhash32", "xxhash64", "murmur32", "spookyhash",
                      "blake3", "crc32c", "xxh3_64", "xxh3_128"),
       serialize=TRUE, file=FALSE,
       length=Inf, skip="auto", ascii=FALSE, raw=FALSE, seed=0,
       errormode=c("stop","warn","silent"),
       serializeVersion=.getSerializeVersion())
}
\arguments{
  \item{object}{An arbitrary R object which will then be passed to the
    \code{\link{serialize}} function, unless the \code{serialize}
    argument is set to \code{FALSE}.}
  \item{algo}{The algorithms to be used; currently available choices are
    \code{md5}, which is also the default, \code{sha1}, \code{crc32},
    \code{sha256}, \code{sha512}, \code{xxhash32}, \code{xxhash64},
    \code{murmur32}, \code{spookyhash}, \code{blake3}, \code{crc32c},
    \code{xxh3_64}, and \code{xxh3_128}.}
  \item{serialize}{A logical variable indicating whether the object
    should be serialized using \code{serialize} (in ASCII
    form). Setting this to \code{FALSE} allows to compare the digest
    output of given character strings to known control output. It also
    allows the use of raw vectors such as the output of non-ASCII
    serialization.
  }
  \item{file}{A logical variable indicating whether the object is a file
    name or a file name if \code{object} is not specified.}
  \item{length}{Number of characters to process. By default, when
    \code{length} is set to \code{Inf}, the whole string or file is
    processed.}
  \item{skip}{Number of input bytes to skip before calculating the
    digest. Negative values are invalid and currently treated as zero.
    Special value \code{"auto"} will cause serialization header to be
    skipped if \code{serialize} is set to \code{TRUE} (the serialization
    header contains the R version number thus skipping it allows the
    comparison of hashes across platforms and some R versions).
  }
  \item{ascii}{This flag is passed to the \code{serialize} function if
    \code{serialize} is set to \code{TRUE}, determining whether the hash
    is computed on the ASCII or binary representation.}
  \item{raw}{A logical variable with a default value of FALSE, implying
    \code{digest} returns digest output as ASCII hex values. Set to TRUE
    to return \code{digest} output in raw (binary) form. Note that this
    option is supported by most but not all of the implemented hashing
    algorithms}
  \item{seed}{an integer to seed the random number generator.  This is only
    used in the \code{xxhash32}, \code{xxhash64} and \code{murmur32} functions
    and can be used to generate additional hashes for the same input if
    desired.}
  \item{errormode}{A character value denoting a choice for the behaviour in
    the case of error: \sQuote{stop} aborts (and is the default value),
    \sQuote{warn} emits a warning and returns \code{NULL} and
    \sQuote{silent} suppresses the error and returns an empty string.}
  \item{serializeVersion}{An integer value specifying the internal
    version of the serialization format, with 2 being the default;
    see \code{\link{serialize}} for details. The \code{serializeVersion}
    field of \code{\link{option}} can also be used to set a different
    value.}
}
\value{
  The \code{digest} function returns a character string of a fixed
  length containing the requested digest of the supplied R object. This
  string is of length 32 for MD5; of length 40 for SHA-1; of length 8
  for CRC32 a string; of length 8 for for xxhash32; of length 16 for
  xxhash64; and of length 8 for murmur32.
}
\details{
  Cryptographic hash functions are well researched and documented. The
  MD5 algorithm by Ron Rivest is specified in RFC 1321. The SHA-1
  algorithm is specified in FIPS-180-1, SHA-2 is described in
  FIPS-180-2.

  For md5, sha-1 and sha-256, this R implementation relies on standalone
  implementations in C by Christophe Devine. For crc32, code from the
  zlib library by Jean-loup Gailly and Mark Adler is used.

  For sha-512, a standalone implementation from Aaron Gifford is used.

  For xxhash32, xxhash64, xxh3_64 and xxh3_128 the reference implementation
  by Yann Collet is used.

  For murmur32, the progressive implementation by Shane Day is used.

  For spookyhash, the original source code by Bob Jenkins is used. The R implementation
  that integrates R's serialization directly with the algorithm allowing for
  memory-efficient incremental calculation of the hash is by Gabe Becker.

  For blake3, the C implementation by Samuel Neves and Jack O'Connor is used.

  For crc32c, the portable (i.e. non-hardware accelerated) version from
  Google is used.

  Please note that this package is not meant to be used for
  cryptographic purposes for which more comprehensive (and widely
  tested) libraries such as OpenSSL should be used. Also, it is known
  that crc32 is not collision-proof. For sha-1, recent results indicate
  certain cryptographic weaknesses as well. For more details, see for example
  \url{https://www.schneier.com/blog/archives/2005/02/cryptanalysis_o.html}.
}
\section{Change Management}{
  Version 0.6.16 of digest corrects an error in which \code{crc32} was not
  guaranteeing an eight-character return. We now pad with zero to always
  return eight characters. Should the previous behaviour be required,
  set \code{option("digestOldCRC32Format"=TRUE)} and the output will be
  consistent with prior version (but not be consistentnly eight characters).
}
\references{
  MD5: \url{https://www.ietf.org/rfc/rfc1321.txt}.

  SHA-1: \url{https://en.wikipedia.org/wiki/SHA-1}.
  SHA-256: \url{https://csrc.nist.gov/publications/fips/fips180-2/fips180-2withchangenotice.pdf}.
  CRC32:  The original reference webpage at \code{rocksoft.com} has
  vanished from the web; see
  \url{https://en.wikipedia.org/wiki/Cyclic_redundancy_check} for
  general information on CRC algorithms.

  \url{https://aarongifford.com/computers/sha.html} for the
  integrated C implementation of sha-512.

  The page for the code underlying the C functions used here for sha-1
  and md5, and further references, is no longer accessible.  Please see
  \url{https://en.wikipedia.org/wiki/SHA-1} and
  \url{https://en.wikipedia.org/wiki/MD5}.

  \url{https://zlib.net} for documentation on the zlib library which
  supplied the code for crc32.

  \url{https://en.wikipedia.org/wiki/SHA_hash_functions} for
  documentation on the sha functions.

  \url{https://github.com/Cyan4973/xxHash} for documentation on the xxHash
  functions.

  \url{https://github.com/aappleby/smhasher} for documentation on MurmurHash.

  \url{https://burtleburtle.net/bob/hash/spooky.html} for the original source code of SpookyHash.

  \url{https://github.com/BLAKE3-team/BLAKE3/} for the original source code of blake3.

  \url{https://github.com/google/crc32c} for the (non-hardware-accelerated) crc32c code.
}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  Antoine Lucas for the integration of crc32; Jarek Tuszynski for the
  file-based operations; Henrik Bengtsson and Simon Urbanek for improved
  serialization patches; Christophe Devine for the hash function
  implementations for sha-1, sha-256 and md5; Jean-Loup Gailly and Mark Adler
  for crc32; Hannes Muehleisen for the integration of sha-512; Jim Hester for
  the integration of xxhash32, xxhash64 and murmur32; Kendon Bell for
  the integration of spookyhash using Gabe Becker's R package fastdigest.}
\seealso{\code{\link{serialize}}, \code{\link{md5sum}}}

\examples{

## Standard RFC 1321 test vectors
md5Input <-
  c("",
    "a",
    "abc",
    "message digest",
    "abcdefghijklmnopqrstuvwxyz",
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
    paste("12345678901234567890123456789012345678901234567890123456789012",
          "345678901234567890", sep=""))
md5Output <-
  c("d41d8cd98f00b204e9800998ecf8427e",
    "0cc175b9c0f1b6a831c399e269772661",
    "900150983cd24fb0d6963f7d28e17f72",
    "f96b697d7cb7938d525a2f31aaf161d0",
    "c3fcd3d76192e4007dfb496cca67e13b",
    "d174ab98d277d9f5a5611c2c9f419d9f",
    "57edf4a22be3c955ac49da2e2107b67a")

for (i in seq(along=md5Input)) {
  md5 <- digest(md5Input[i], serialize=FALSE)
  stopifnot(identical(md5, md5Output[i]))
}

sha1Input <-
  c("abc", "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
sha1Output <-
  c("a9993e364706816aba3e25717850c26c9cd0d89d",
    "84983e441c3bd26ebaae4aa1f95129e5e54670f1")

for (i in seq(along=sha1Input)) {
  sha1 <- digest(sha1Input[i], algo="sha1", serialize=FALSE)
  stopifnot(identical(sha1, sha1Output[i]))
}

crc32Input <-
  c("abc",
    "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
crc32Output <-
  c("352441c2",
    "171a3f5f")

for (i in seq(along=crc32Input)) {
  crc32 <- digest(crc32Input[i], algo="crc32", serialize=FALSE)
  stopifnot(identical(crc32, crc32Output[i]))
}


sha256Input <-
  c("abc",
    "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
sha256Output <-
  c("ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad",
    "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1")

for (i in seq(along=sha256Input)) {
  sha256 <- digest(sha256Input[i], algo="sha256", serialize=FALSE)
  stopifnot(identical(sha256, sha256Output[i]))
}

# SHA 512 example
sha512Input <-
  c("abc",
    "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
sha512Output <-
  c(paste("ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a",
          "2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f",
          sep=""),
    paste("204a8fc6dda82f0a0ced7beb8e08a41657c16ef468b228a8279be331a703c335",
          "96fd15c13b1b07f9aa1d3bea57789ca031ad85c7a71dd70354ec631238ca3445",
          sep=""))

for (i in seq(along=sha512Input)) {
  sha512 <- digest(sha512Input[i], algo="sha512", serialize=FALSE)
  stopifnot(identical(sha512, sha512Output[i]))
}

## xxhash32 example
xxhash32Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
xxhash32Output <-
    c("32d153ff",
      "89ea60c3",
      "02cc5d05")

for (i in seq(along=xxhash32Input)) {
    xxhash32 <- digest(xxhash32Input[i], algo="xxhash32", serialize=FALSE)
    cat(xxhash32, "\n")
    stopifnot(identical(xxhash32, xxhash32Output[i]))
}

## xxhash64 example
xxhash64Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
xxhash64Output <-
    c("44bc2cf5ad770999",
      "f06103773e8585df",
      "ef46db3751d8e999")

for (i in seq(along=xxhash64Input)) {
    xxhash64 <- digest(xxhash64Input[i], algo="xxhash64", serialize=FALSE)
    cat(xxhash64, "\n")
    stopifnot(identical(xxhash64, xxhash64Output[i]))
}

## these outputs were calculated using mmh3 python package
murmur32Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
murmur32Output <-
    c("b3dd93fa",
      "ee925b90",
      "00000000")

for (i in seq(along=murmur32Input)) {
    murmur32 <- digest(murmur32Input[i], algo="murmur32", serialize=FALSE)
    cat(murmur32, "\n")
    stopifnot(identical(murmur32, murmur32Output[i]))
}

## these outputs were calculated using spooky python package
spookyInput <-
    c("a",
      "abc",
      "message digest")
spookyOutput <-
    c("bdc9bba09181101a922a4161f0584275",
      "67c93775f715ab8ab01178caf86713c6",
      "9630c2a55c0987a0db44434f9d67a192")

for (i in seq(along=spookyInput)) {
    # skip = 30 skips the serialization header and just hashes the strings
    spooky <- digest(spookyInput[i], algo="spookyhash", skip = 30)
    cat(spooky, "\n")
    ## we can only compare to reference output on little-endian systems
    if (isTRUE(.Call(digest:::is_little_endian)))
        stopifnot(identical(spooky, spookyOutput[i]))
}

## blake3 example
blake3Input <-
    c("abc",
      "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
      "")
blake3Output <-
    c("6437b3ac38465133ffb63b75273a8db548c558465d79db03fd359c6cd5bd9d85",
      "c19012cc2aaf0dc3d8e5c45a1b79114d2df42abb2a410bf54be09e891af06ff8",
      "af1349b9f5f9a1a6a0404dea36dcc9499bcb25c9adc112b7cc9a93cae41f3262")

for (i in seq(along=blake3Input)) {
    blake3 <- digest(blake3Input[i], algo="blake3", serialize=FALSE)
    cat(blake3, "\n")
    stopifnot(identical(blake3, blake3Output[i]))
}

## crc32c
crc32cInput <- c("123456789", "The quick brown fox jumps over the lazy dog")
crc32cOutput <- c("e3069283", "22620404")
for (i in seq_along(crc32cInput)) {
    crc32c <- digest(crc32cInput[i], algo="crc32c", serialize=FALSE)
    cat(crc32c, "\n")
    stopifnot(identical(crc32c, crc32cOutput[i]))
}

# example of a digest of a standard R list structure
digest(list(LETTERS, data.frame(a=letters[1:5], b=matrix(1:10,ncol=2))))

# test 'length' parameter and file input
fname <- file.path(R.home(),"COPYING")
x <- readChar(fname, file.info(fname)$size) # read file
for (alg in c("sha1", "md5", "crc32")) {
  # partial file
  h1 <- digest(x    , length=18000, algo=alg, serialize=FALSE)
  h2 <- digest(fname, length=18000, algo=alg, serialize=FALSE, file=TRUE)
  h3 <- digest( substr(x,1,18000) , algo=alg, serialize=FALSE)
  stopifnot( identical(h1,h2), identical(h1,h3) )
  # whole file
  h1 <- digest(x    , algo=alg, serialize=FALSE)
  h2 <- digest(fname, algo=alg, serialize=FALSE, file=TRUE)
  stopifnot( identical(h1,h2) )
}

# compare md5 algorithm to other tools
library(tools)
fname <- file.path(R.home(),"COPYING")
h1 <- as.character(md5sum(fname))
h2 <- digest(fname, algo="md5", file=TRUE)
stopifnot( identical(h1,h2) )

## digest is _designed_ to return one has summary per object to for a desired
## For vectorised output see digest::getVDigest() which provides
## better performance than base::Vectorize()

md5 <- getVDigest()
v <- md5(1:5)     			# digest integers 1 to 5
stopifnot(identical(v[1], digest(1L)),	# check first and third result
          identical(v[3], digest(3L)))

}
\keyword{misc}
